#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.czechia import Czechia
from tests.common import CommonCountryTests


class TestCzechia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Czechia)

    def test_new_years_day(self):
        self.assertHolidayName(
            "Nový rok", (f"{year}-01-01" for year in range(self.start_year, 2000))
        )

    def test_independent_czech_state_day(self):
        name = "Den obnovy samostatného českého státu"
        self.assertHolidayName(name, (f"{year}-01-01" for year in range(2000, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2000))

    def test_good_friday(self):
        name = "Velký pátek"
        self.assertHolidayName(
            name,
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
        )
        self.assertHolidayName(name, range(2016, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2016))

    def test_easter_monday(self):
        name = "Velikonoční pondělí"
        self.assertHolidayName(
            name,
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        self.assertHolidayName("Svátek práce", (f"{year}-05-01" for year in self.full_range))

    def test_victory_day(self):
        name_1952 = "Den osvobození od fašismu"
        name_2001 = "Den osvobození"
        name_2004 = "Den vítězství"
        self.assertHolidayName(
            name_1952,
            (f"{year}-05-09" for year in range(self.start_year, 1992)),
            (f"{year}-05-08" for year in range(1992, 2001)),
        )
        self.assertHolidayName(name_2001, (f"{year}-05-08" for year in range(2001, 2004)))
        self.assertHolidayName(name_2004, (f"{year}-05-08" for year in range(2004, self.end_year)))
        self.assertNoHolidayName(name_1952, range(2001, self.end_year))
        self.assertNoHolidayName(
            name_2001, range(self.start_year, 2001), range(2004, self.end_year)
        )
        self.assertNoHolidayName(name_2004, range(self.start_year, 2004))

    def test_cyril_and_methodius_day(self):
        name = "Den slovanských věrozvěstů Cyrila a Metoděje"
        self.assertHolidayName(name, (f"{year}-07-05" for year in range(1990, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1990))

    def test_jan_hus_day(self):
        name = "Den upálení mistra Jana Husa"
        self.assertHolidayName(name, (f"{year}-07-06" for year in range(1990, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1990))

    def test_statehood_day(self):
        name = "Den české státnosti"
        self.assertHolidayName(name, (f"{year}-09-28" for year in range(2000, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2000))

    def test_independent_czechoslovak_state_day(self):
        self.assertHolidayName(
            "Den vzniku samostatného československého státu",
            (f"{year}-10-28" for year in self.full_range),
        )

    def test_struggle_for_freedom_and_democracy_day(self):
        name_1990 = "Den boje za svobodu a demokracii"
        name_2019 = "Den boje za svobodu a demokracii a Mezinárodní den studentstva"
        self.assertHolidayName(name_1990, (f"{year}-11-17" for year in range(1990, 2019)))
        self.assertHolidayName(name_2019, (f"{year}-11-17" for year in range(2019, self.end_year)))
        self.assertNoHolidayName(
            name_1990, range(self.start_year, 1990), range(2019, self.end_year)
        )
        self.assertNoHolidayName(name_2019, range(self.start_year, 2019))

    def test_christmas_eve(self):
        name = "Štědrý den"
        self.assertHolidayName(name, (f"{year}-12-24" for year in range(1990, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1990))

    def test_christmas_day(self):
        self.assertHolidayName("1. svátek vánoční", (f"{year}-12-25" for year in self.full_range))

    def test_christmas_second_day(self):
        self.assertHolidayName("2. svátek vánoční", (f"{year}-12-26" for year in self.full_range))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Den obnovy samostatného českého státu; Nový rok"),
            ("2022-04-15", "Velký pátek"),
            ("2022-04-18", "Velikonoční pondělí"),
            ("2022-05-01", "Svátek práce"),
            ("2022-05-08", "Den vítězství"),
            ("2022-07-05", "Den slovanských věrozvěstů Cyrila a Metoděje"),
            ("2022-07-06", "Den upálení mistra Jana Husa"),
            ("2022-09-28", "Den české státnosti"),
            ("2022-10-28", "Den vzniku samostatného československého státu"),
            ("2022-11-17", "Den boje za svobodu a demokracii a Mezinárodní den studentstva"),
            ("2022-12-24", "Štědrý den"),
            ("2022-12-25", "1. svátek vánoční"),
            ("2022-12-26", "2. svátek vánoční"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "Independent Czech State Restoration Day; New Year's Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-08", "Victory Day"),
            ("2022-07-05", "Saints Cyril and Methodius Day"),
            ("2022-07-06", "Jan Hus Day"),
            ("2022-09-28", "Statehood Day"),
            ("2022-10-28", "Independent Czechoslovak State Day"),
            (
                "2022-11-17",
                "Struggle for Freedom and Democracy Day and International Students' Day",
            ),
            ("2022-12-24", "Christmas Eve"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Second Day of Christmas"),
        )

    def test_l10n_sk(self):
        self.assertLocalizedHolidays(
            "sk",
            ("2022-01-01", "Deň obnovy samostatného českého štátu; Nový rok"),
            ("2022-04-15", "Veľký piatok"),
            ("2022-04-18", "Veľkonočný pondelok"),
            ("2022-05-01", "Sviatok práce"),
            ("2022-05-08", "Deň víťazstva"),
            ("2022-07-05", "Deň slovanských vierozvestcov Cyrila a Metoda"),
            ("2022-07-06", "Deň upálenia majstra Jána Husa"),
            ("2022-09-28", "Deň českej štátnosti"),
            ("2022-10-28", "Deň vzniku samostatného československého štátu"),
            ("2022-11-17", "Deň boja za slobodu a demokraciu a Medzinárodný deň študentstva"),
            ("2022-12-24", "Štedrý deň"),
            ("2022-12-25", "1. sviatok vianočný"),
            ("2022-12-26", "2. sviatok vianočný"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "День відновлення незалежної чеської держави; Новий рік"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-18", "Великодній понеділок"),
            ("2022-05-01", "День праці"),
            ("2022-05-08", "День Перемоги"),
            ("2022-07-05", "День Святих Кирила та Мефодія"),
            ("2022-07-06", "День спалення Яна Гуса"),
            ("2022-09-28", "День чеської державності"),
            ("2022-10-28", "День створення незалежної чехословацької держави"),
            ("2022-11-17", "День боротьби за свободу і демократію та Міжнародний день студентів"),
            ("2022-12-24", "Святий вечір"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "Другий день Різдва"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.cook_islands import CookIslands
from tests.common import CommonCountryTests


class TestCookIslands(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(CookIslands)

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2011-01-03",
            "2012-01-03",
            "2017-01-03",
            "2022-01-03",
            "2023-01-03",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_day_after_new_years(self):
        name = "Day after New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-02" for year in self.full_range))
        obs_dts = (
            "2010-01-04",
            "2011-01-04",
            "2016-01-04",
            "2021-01-04",
            "2022-01-04",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_anzac_day(self):
        self.assertHolidayName("Anzac Day", (f"{year}-04-25" for year in self.full_range))

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_sovereigns_birthday(self):
        name = "Sovereign's Birthday"
        self.assertHolidayName(
            name,
            "2020-06-01",
            "2021-06-07",
            "2022-06-06",
            "2023-06-05",
            "2024-06-03",
            "2025-06-02",
        )
        self.assertHolidayName(name, self.full_range)

    def test_house_of_ariki_day(self):
        name = "Ra o te Ui Ariki"
        self.assertHolidayName(
            name,
            "2012-06-06",
            "2020-07-03",
            "2021-07-02",
            "2022-07-01",
            "2023-07-07",
            "2024-07-05",
            "2025-07-04",
        )
        self.assertHolidayName(name, range(2012, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2012))

    def test_constitution_day(self):
        name = "Constitution Day"
        self.assertHolidayName(name, (f"{year}-08-04" for year in self.full_range))
        obs_dts = (
            "2012-08-06",
            "2013-08-05",
            "2018-08-06",
            "2019-08-05",
            "2024-08-05",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_cook_islands_gospel_day(self):
        name = "Cook Islands Gospel Day"
        self.assertHolidayName(name, (f"{year}-10-26" for year in self.full_range))
        obs_dts = (
            "2013-10-28",
            "2014-10-27",
            "2019-10-28",
            "2024-10-28",
            "2025-10-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2010-12-27",
            "2011-12-27",
            "2016-12-27",
            "2021-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dts = (
            "2009-12-28",
            "2010-12-28",
            "2015-12-28",
            "2020-12-28",
            "2021-12-28",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_penrhyn_gospel_day(self):
        name = "Penrhyn Gospel Day"
        self.assertHolidayName(name, (f"{year}-03-13" for year in range(self.start_year, 2012)))
        self.assertNoHolidayName(name, range(2012, self.end_year))
        obs_dts = (
            "2004-03-15",
            "2005-03-14",
            "2010-03-15",
            "2011-03-14",
        )
        name_observed = f"{name} (observed)"
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNoHolidayName(name_observed, range(2012, self.end_year))

    def test_palmerston_gospel_day(self):
        name = "Palmerston Gospel Day"
        self.assertHolidayName(name, (f"{year}-05-25" for year in range(self.start_year, 2012)))
        self.assertNoHolidayName(name, range(2012, self.end_year))
        obs_dts = (
            "2002-05-27",
            "2003-05-26",
            "2008-05-26",
        )
        name_observed = f"{name} (observed)"
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNoHolidayName(name_observed, range(2012, self.end_year))

    def test_mangaia_gospel_day(self):
        name = "Mangaia Gospel Day"
        self.assertHolidayName(name, (f"{year}-06-15" for year in range(self.start_year, 2012)))
        self.assertNoHolidayName(name, range(2012, self.end_year))
        obs_dts = (
            "2002-06-17",
            "2003-06-16",
            "2008-06-16",
        )
        name_observed = f"{name} (observed)"
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNoHolidayName(name_observed, range(2012, self.end_year))

    def test_atiu_gospel_day(self):
        name = "Atiu Gospel Day"
        self.assertHolidayName(name, (f"{year}-07-20" for year in range(self.start_year, 2012)))
        self.assertNoHolidayName(name, range(2012, self.end_year))
        obs_dts = (
            "2002-07-22",
            "2003-07-21",
            "2008-07-21",
        )
        name_observed = f"{name} (observed)"
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHolidayName(name_observed, obs_dts)
        self.assertNoHolidayName(name_observed, range(2012, self.end_year))

    def test_mitiaro_gospel_day(self):
        name = "Mitiaro Gospel Day"
        self.assertHolidayName(name, (f"{year}-07-21" for year in range(self.start_year, 2012)))
        self.assertNoHolidayName(name, range(2012, self.end_year))
        obs_dts = (
            "2001-07-23",
            "2002-07-22",
            "2007-07-23",
        )
        name_observed = f"{name} (observed)"
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHolidayName(name_observed, obs_dts)
        self.assertNoHolidayName(name_observed, range(2012, self.end_year))

    def test_mauke_gospel_day(self):
        name = "Mauke Gospel Day"
        self.assertHolidayName(name, (f"{year}-07-23" for year in range(self.start_year, 2012)))
        self.assertNoHolidayName(name, range(2012, self.end_year))
        obs_dts = (
            "2000-07-24",
            "2005-07-26",
            "2011-07-26",
        )
        name_observed = f"{name} (observed)"
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNoHolidayName(name_observed, range(2012, self.end_year))

    def test_rarotonga_gospel_day(self):
        name = "Rarotonga Gospel Day"
        self.assertHolidayName(name, (f"{year}-07-25" for year in range(self.start_year, 2012)))
        self.assertNoHolidayName(name, range(2012, self.end_year))
        obs_dts = (
            "2004-07-26",
            "2009-07-27",
            "2010-07-26",
        )
        name_observed = f"{name} (observed)"
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNoHolidayName(name_observed, range(2012, self.end_year))

    def test_manihiki_gospel_day(self):
        name = "Manihiki Gospel Day"
        self.assertHolidayName(name, (f"{year}-08-08" for year in range(self.start_year, 2012)))
        self.assertNoHolidayName(name, range(2012, self.end_year))
        obs_dts = (
            "2004-08-09",
            "2009-08-10",
            "2010-08-09",
        )
        name_observed = f"{name} (observed)"
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNoHolidayName(name_observed, range(2012, self.end_year))

    def test_rakahanga_gospel_day(self):
        name = "Rakahanga Gospel Day"
        self.assertHolidayName(name, (f"{year}-08-15" for year in range(self.start_year, 2012)))
        self.assertNoHolidayName(name, range(2012, self.end_year))
        obs_dts = (
            "2004-08-16",
            "2009-08-17",
            "2010-08-16",
        )
        name_observed = f"{name} (observed)"
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNoHolidayName(name_observed, range(2012, self.end_year))

    def test_aitutaki_gospel_day(self):
        name = "Aitutaki Gospel Day"
        self.assertHolidayName(name, (f"{year}-10-27" for year in range(self.start_year, 2012)))
        self.assertNoHolidayName(name, range(2012, self.end_year))
        obs_dts = (
            "2001-10-29",
            "2002-10-28",
            "2007-10-29",
        )
        name_observed = f"{name} (observed)"
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNoHolidayName(name_observed, range(2012, self.end_year))

    def test_pukapuka_gospel_day(self):
        name = "Pukapuka Gospel Day"
        self.assertHolidayName(name, (f"{year}-12-08" for year in range(self.start_year, 2012)))
        self.assertNoHolidayName(name, range(2012, self.end_year))
        obs_dts = (
            "2001-12-10",
            "2002-12-09",
            "2007-12-10",
        )
        name_observed = f"{name} (observed)"
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNoHolidayName(name_observed, range(2012, self.end_year))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "New Year's Day"),
            ("2022-01-02", "Day after New Year's Day"),
            ("2022-01-03", "New Year's Day (observed)"),
            ("2022-01-04", "Day after New Year's Day (observed)"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-04-25", "Anzac Day"),
            ("2022-06-06", "Sovereign's Birthday"),
            ("2022-07-01", "Ra o te Ui Ariki"),
            ("2022-08-04", "Constitution Day"),
            ("2022-10-26", "Cook Islands Gospel Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
            ("2022-12-27", "Christmas Day (observed)"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-02", "Day after New Year's Day"),
            ("2022-01-03", "New Year's Day (observed)"),
            ("2022-01-04", "Day after New Year's Day (observed)"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-04-25", "Anzac Day"),
            ("2022-06-06", "Sovereign's Birthday"),
            ("2022-07-01", "Day of the House of Ariki"),
            ("2022-08-04", "Constitution Day"),
            ("2022-10-26", "Cook Islands Gospel Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
            ("2022-12-27", "Christmas Day (observed)"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

"""Generate docs/changelog.md from CHANGES.md"""

import re
from pathlib import Path

import mkdocs_gen_files


def main():
    changes_file = Path(__file__).parents[2] / "CHANGES.md"
    file_content = changes_file.read_text(encoding="utf-8")
    file_content = re.sub(
        r"#(\d+)", r"[\g<0>](https://github.com/vacanza/holidays/pull/\1)", file_content
    )

    with mkdocs_gen_files.open("changelog.md", "w", newline="\n") as f:
        f.write("\n".join(("# Changelog", "", *file_content.split("\n"))))


if __name__ in {"__main__", "<run_path>"}:
    main()



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.serbia import Serbia
from tests.common import CommonCountryTests


class TestSerbia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Serbia)

    def test_new_years_day(self):
        name = "Нова година"
        self.assertHolidayName(
            name,
            (f"{year}-01-01" for year in self.full_range),
            (f"{year}-01-02" for year in self.full_range),
        )
        obs_dts = (
            "2017-01-03",
            "2022-01-03",
            "2023-01-03",
        )
        self.assertHolidayName(f"{name} (слободан дан)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_orthodox_christmas(self):
        self.assertHolidayName("Божић", (f"{year}-01-07" for year in self.full_range))

    def test_statehood_day(self):
        name = "Дан државности Србије"
        self.assertHolidayName(
            name,
            (f"{year}-02-15" for year in self.full_range),
            (f"{year}-02-16" for year in self.full_range),
        )
        obs_dts = (
            "2015-02-17",
            "2020-02-17",
            "2025-02-17",
        )
        self.assertHolidayName(f"{name} (слободан дан)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_orthodox_good_friday(self):
        name = "Велики петак"
        self.assertHolidayName(
            name,
            "2020-04-17",
            "2021-04-30",
            "2022-04-22",
            "2023-04-14",
            "2024-05-03",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_orthodox_holy_saturday(self):
        name = "Велика субота"
        self.assertHolidayName(
            name,
            "2020-04-18",
            "2021-05-01",
            "2022-04-23",
            "2023-04-15",
            "2024-05-04",
            "2025-04-19",
        )
        self.assertHolidayName(name, self.full_range)

    def test_orthodox_easter_sunday(self):
        name = "Васкрс"
        self.assertHolidayName(
            name,
            "2020-04-19",
            "2021-05-02",
            "2022-04-24",
            "2023-04-16",
            "2024-05-05",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)

    def test_orthodox_easter_monday(self):
        name = "Други дан Васкрса"
        self.assertHolidayName(
            name,
            "2020-04-20",
            "2021-05-03",
            "2022-04-25",
            "2023-04-17",
            "2024-05-06",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        name = "Празник рада"
        self.assertHolidayName(
            name,
            (f"{year}-05-01" for year in self.full_range),
            (f"{year}-05-02" for year in self.full_range),
        )
        obs_dts = (
            "2016-05-03",
            "2021-05-04",
            "2022-05-03",
        )
        self.assertHolidayName(f"{name} (слободан дан)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_armistice_day(self):
        name = "Дан примирја у Првом светском рату"
        self.assertHolidayName(name, (f"{year}-11-11" for year in self.full_range))
        obs_dts = (
            "2012-11-12",
            "2018-11-12",
        )
        self.assertHolidayName(f"{name} (слободан дан)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Нова година"),
            ("2022-01-02", "Нова година"),
            ("2022-01-03", "Нова година (слободан дан)"),
            ("2022-01-07", "Божић"),
            ("2022-02-15", "Дан државности Србије"),
            ("2022-02-16", "Дан државности Србије"),
            ("2022-04-22", "Велики петак"),
            ("2022-04-23", "Велика субота"),
            ("2022-04-24", "Васкрс"),
            ("2022-04-25", "Други дан Васкрса"),
            ("2022-05-01", "Празник рада"),
            ("2022-05-02", "Празник рада"),
            ("2022-05-03", "Празник рада (слободан дан)"),
            ("2022-11-11", "Дан примирја у Првом светском рату"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-02", "New Year's Day"),
            ("2022-01-03", "New Year's Day (observed)"),
            ("2022-01-07", "Orthodox Christmas Day"),
            ("2022-02-15", "Statehood Day"),
            ("2022-02-16", "Statehood Day"),
            ("2022-04-22", "Good Friday"),
            ("2022-04-23", "Holy Saturday"),
            ("2022-04-24", "Easter Sunday"),
            ("2022-04-25", "Easter Monday"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-02", "Labor Day"),
            ("2022-05-03", "Labor Day (observed)"),
            ("2022-11-11", "Armistice Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.equatorial_guinea import EquatorialGuinea
from tests.common import CommonCountryTests


class TestEquatorialGuinea(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(EquatorialGuinea)

    def test_special_holidays(self):
        self.assertHoliday(
            "2024-01-23",
        )

    def test_new_years_day(self):
        name = "Año Nuevo"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2011-01-03",
            "2012-01-02",
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_international_womens_day(self):
        self.assertHolidayName(
            "Día Internacional de la Mujer", (f"{year}-03-08" for year in self.full_range)
        )

    def test_maundy_thursday(self):
        name = "Jueves Santo"
        self.assertHolidayName(
            name,
            "2020-04-09",
            "2021-04-01",
            "2022-04-14",
            "2023-04-06",
            "2024-03-28",
            "2025-04-17",
        )
        self.assertHolidayName(name, self.full_range)

    def test_good_friday(self):
        name = "Viernes Santo"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_international_labor_day(self):
        name = "Día Internacional del Trabajo"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dts = (
            "2010-05-03",
            "2011-05-02",
            "2016-05-02",
            "2021-05-03",
            "2022-05-02",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_african_liberation_day(self):
        self.assertHolidayName(
            "Día de la liberación Africana", (f"{year}-05-25" for year in self.full_range)
        )

    def test_presidents_day(self):
        name = "Natalicio de Su Excelencia el Presidente de la República"
        self.assertHolidayName(name, (f"{year}-06-05" for year in self.full_range))
        obs_dts = (
            "2010-06-07",
            "2011-06-06",
            "2016-06-06",
            "2021-06-07",
            "2022-06-06",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_corpus_christi(self):
        name = "Corpus Christi"
        self.assertHolidayName(
            name,
            "2020-06-11",
            "2021-06-03",
            "2022-06-16",
            "2023-06-08",
            "2024-05-30",
            "2025-06-19",
        )
        self.assertHolidayName(name, self.full_range)

    def test_armed_forces_day(self):
        name = "Día de las Fuerzas Armadas"
        self.assertHolidayName(name, (f"{year}-08-03" for year in self.full_range))
        obs_dts = (
            "2008-08-04",
            "2013-08-05",
            "2014-08-04",
            "2019-08-05",
            "2024-08-05",
            "2025-08-04",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_constitution_day(self):
        name = "Día de la Constitución"
        self.assertHolidayName(name, (f"{year}-08-15" for year in self.full_range))
        obs_dts = (
            "2009-08-17",
            "2010-08-16",
            "2015-08-17",
            "2020-08-17",
            "2021-08-16",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_day(self):
        name = "Día de la Independencia Nacional"
        self.assertHolidayName(name, (f"{year}-10-12" for year in self.full_range))
        obs_dts = (
            "2008-10-13",
            "2013-10-14",
            "2014-10-13",
            "2019-10-14",
            "2024-10-14",
            "2025-10-13",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_immaculate_conception(self):
        name = "Festividad de la Inmaculada Concepción de María"
        self.assertHolidayName(name, (f"{year}-12-08" for year in self.full_range))
        obs_dts = (
            "2007-12-10",
            "2012-12-10",
            "2013-12-09",
            "2018-12-10",
            "2019-12-09",
            "2024-12-09",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        name = "Día de Navidad"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2010-12-27",
            "2011-12-26",
            "2016-12-26",
            "2021-12-27",
            "2022-12-26",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_subdivision_holidays(self):
        name = "Fiesta Patronal de Annobón"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "AN":
                self.assertHolidayName(
                    name, holidays, (f"{year}-06-13" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "Año Nuevo"),
            ("2025-03-08", "Día Internacional de la Mujer"),
            ("2025-04-17", "Jueves Santo"),
            ("2025-04-18", "Viernes Santo"),
            ("2025-05-01", "Día Internacional del Trabajo"),
            ("2025-05-25", "Día de la liberación Africana"),
            ("2025-06-05", "Natalicio de Su Excelencia el Presidente de la República"),
            ("2025-06-13", "Fiesta Patronal de Annobón"),
            ("2025-06-19", "Corpus Christi"),
            ("2025-08-03", "Día de las Fuerzas Armadas"),
            ("2025-08-04", "Día de las Fuerzas Armadas (observado)"),
            ("2025-08-15", "Día de la Constitución"),
            ("2025-10-12", "Día de la Independencia Nacional"),
            ("2025-10-13", "Día de la Independencia Nacional (observado)"),
            ("2025-12-08", "Festividad de la Inmaculada Concepción de María"),
            ("2025-12-25", "Día de Navidad"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-03-08", "International Women's Day"),
            ("2025-04-17", "Maundy Thursday"),
            ("2025-04-18", "Good Friday"),
            ("2025-05-01", "International Labor Day"),
            ("2025-05-25", "African Liberation Day"),
            ("2025-06-05", "President's Day"),
            ("2025-06-13", "Patron Saint Festival of Annobón"),
            ("2025-06-19", "Corpus Christi"),
            ("2025-08-03", "Armed Forces Day"),
            ("2025-08-04", "Armed Forces Day (observed)"),
            ("2025-08-15", "Constitution Day"),
            ("2025-10-12", "Independence Day"),
            ("2025-10-13", "Independence Day (observed)"),
            ("2025-12-08", "Immaculate Conception"),
            ("2025-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import ARMED_FORCES, BANK, GOVERNMENT, SCHOOL
from holidays.countries.thailand import Thailand
from tests.common import CommonCountryTests


class TestThailand(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Thailand)

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(
            Thailand(categories=ARMED_FORCES, years=range(self.start_year, 1959))
        )
        self.assertNoHolidays(
            Thailand(
                categories=BANK, years=(*range(self.start_year, 1943), *range(2022, self.end_year))
            )
        )
        self.assertNoHolidays(Thailand(categories=GOVERNMENT, years=range(self.start_year, 1960)))
        self.assertNoHolidays(Thailand(categories=SCHOOL, years=range(self.start_year, 1957)))

    def test_special_holidays(self):
        dts = (
            # 1992-1994 (include In Lieus, Checked with Bank of Thailand Data).
            "1992-05-18",
            "1992-12-07",
            "1993-03-08",
            "1993-05-03",
            "1993-10-25",
            "1993-12-06",
            "1994-01-03",
            "1994-05-02",
            "1994-07-25",
            "1994-10-24",
            "1994-12-12",
            # 1995-1997 (Bank of Thailand Data).
            "1996-06-10",
            # 1998-2000 (include In Lieus, Checked with Bank of Thailand Data).
            "1998-05-11",
            "1998-12-07",
            "1999-05-03",
            "1999-05-31",
            "1999-10-25",
            "1999-12-06",
            "2000-01-03",
            "2000-02-21",
            "2000-08-14",
            "2000-12-11",
            "2000-12-29",
            # From 2001 Onwards (Checked with Bank of Thailand Data).
            "2006-04-19",
            "2006-06-09",
            "2006-06-12",
            "2006-06-13",
            "2006-09-20",
            "2009-01-02",
            "2009-04-10",
            "2009-04-16",
            "2009-04-17",
            "2009-07-06",
            "2010-05-20",
            "2010-05-21",
            "2010-08-13",
            "2011-05-16",
            "2011-10-27",
            "2011-10-28",
            "2011-10-29",
            "2011-10-30",
            "2011-10-31",
            "2012-04-09",
            "2013-12-30",
            "2014-08-11",
            "2015-01-02",
            "2015-05-04",
            "2016-05-06",
            "2016-07-18",
            "2016-10-14",
            "2017-10-26",
            "2019-05-06",
            "2020-11-19",
            "2020-11-20",
            "2020-12-11",
            "2021-02-12",
            "2021-04-12",
            "2021-09-24",
            "2022-07-15",
            "2022-07-29",
            "2022-10-14",
            "2022-12-30",
            "2023-05-05",
            "2023-07-31",
            "2023-12-29",
            "2024-04-12",
            "2024-12-30",
            "2025-06-02",
            "2025-08-11",
            "2026-01-02",
        )
        obs_dts = (
            "2007-12-24",
            "2020-07-27",
            "2020-09-04",
            "2020-09-07",
        )
        self.assertHoliday(dts, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_new_years_day(self):
        name = "วันขึ้นปีใหม่"

        self.assertHolidayName(name, (f"{year}-01-01" for year in range(1941, self.end_year)))
        self.assertHolidayName(
            name, (f"{year}-01-02" for year in (*range(1941, 1945), *range(1948, 1954)))
        )
        self.assertNoHolidayName(name, range(self.start_year, 1941))

        self.assertNoNonObservedHoliday(
            "2011-01-03",
            "2012-01-02",
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
            "2028-01-03",
        )

    def test_national_childrens_day(self):
        name = "วันเด็กแห่งชาติ"

        self.assertHolidayName(
            name,
            # First Iteration
            "1955-10-03",
            # Second and Current Iteration
            "2020-01-11",
            "2021-01-09",
            "2022-01-08",
            "2023-01-14",
            "2024-01-13",
            "2025-01-11",
        )
        self.assertNoHolidayName(name, range(self.start_year, 1955), 1964)

    def test_franco_thai_war_armistice_day(self):
        name = "วันลงนามในสัญญาพักรบระหว่างประเทศไทยกับประเทศอินโดจีนฝรั่งเศส"

        self.assertHolidayName(name, (f"{year}-01-28" for year in range(1942, 1945)))
        self.assertNoHolidayName(name, range(self.start_year, 1942), range(1945, self.end_year))

    def test_chakri_memorial_day(self):
        name_1926 = "วันที่ระลึกมหาจักรี"
        name_1938 = "วันจักรี"
        name_1983 = "วันพระบาทสมเด็จพระพุทธยอดฟ้าจุฬาโลกมหาราช และวันที่ระลึกมหาจักรีบรมราชวงศ์"

        self.assertHolidayName(name_1926, (f"{year}-04-06" for year in range(1926, 1938)))
        self.assertHolidayName(name_1938, (f"{year}-04-06" for year in range(1938, 1983)))
        self.assertHolidayName(name_1983, (f"{year}-04-06" for year in range(1983, self.end_year)))
        self.assertNoHolidayName(
            name_1926, range(self.start_year, 1926), range(1938, self.end_year)
        )
        self.assertNoHolidayName(
            name_1938, range(self.start_year, 1938), range(1983, self.end_year)
        )
        self.assertNoHolidayName(name_1983, range(self.start_year, 1983))

        self.assertNoNonObservedHoliday(
            "2013-04-08",
            "2014-04-07",
            "2019-04-08",
            "2024-04-08",
            "2025-04-07",
            "2030-04-08",
        )

    def test_songkran_festival(self):
        name_1914 = "พระราชพิธีตะรุษะสงกรานต์ แลนักขัตฤกษ์"
        name_1926 = "ตะรุษะสงกรานต์"
        name_1938 = "วันตรุษสงกรานต์"
        name_1939 = "วันตรุษสงกรานต์และขึ้นปีใหม่"
        name_1948 = "วันสงกรานต์"

        # MAR 28 - APR 15
        for year in range(self.start_year, 1926):
            self.assertHolidayName(name_1914, (f"{year}-03-28", f"{year}-04-01", f"{year}-04-15"))
        self.assertNoHolidayName(name_1914, range(1926, self.end_year))
        # MAR 31 - APR 3
        for year in range(1926, 1938):
            self.assertHolidayName(name_1926, (f"{year}-03-31", f"{year}-04-01", f"{year}-04-03"))
        self.assertNoHolidayName(
            name_1926, range(self.start_year, 1926), range(1938, self.end_year)
        )
        # MAR 28 - APR 2
        for year in range(1938, 1940):
            self.assertHolidayName(name_1938, (f"{year}-03-31", f"{year}-04-01", f"{year}-04-02"))
        self.assertHolidayName(name_1939, "1940-03-31", "1940-04-01", "1940-04-02")
        self.assertNoHolidayName(
            name_1938, range(self.start_year, 1938), range(1940, self.end_year)
        )
        self.assertNoHolidayName(
            name_1939, range(self.start_year, 1940), range(1941, self.end_year)
        )
        # APR 13-14-15
        for year in (*range(1948, 1954), *range(1998, 2020), *range(2021, self.end_year)):
            self.assertHolidayName(name_1948, (f"{year}-04-13", f"{year}-04-14", f"{year}-04-15"))
        # APR 12-13-14
        for year in range(1989, 1998):
            self.assertHolidayName(name_1948, (f"{year}-04-12", f"{year}-04-13", f"{year}-04-14"))
        # APR 13
        self.assertHolidayName(name_1948, (f"{year}-04-13" for year in range(1957, 1989)))
        # None (2020 is special_public_holidays instead)
        self.assertNoHolidayName(name_1948, "2020-04-13", "2020-04-14", "2020-04-15")
        self.assertNoHolidayName(name_1948, range(self.start_year, 1948), range(1954, 1957))

        self.assertNoNonObservedHoliday(
            "2012-04-16",
            "2013-04-16",
            "2014-04-16",
            "2017-04-17",
            "2018-04-16",
            "2019-04-16",
            # 2020 Songkran Festival special in lieus doesn't counts
            "2023-04-17",
            "2024-04-16",
            "2025-04-16",
            "2028-04-17",
            "2029-04-16",
            "2030-04-16",
        )

    def test_national_labour_day(self):
        name = "วันแรงงานแห่งชาติ"

        self.assertHolidayName(name, (f"{year}-05-01" for year in range(1974, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1974))

        self.assertNoNonObservedHoliday(
            "2010-05-03",
            "2011-05-02",
            "2016-05-02",
            "2021-05-03",
            "2022-05-02",
            "2027-05-03",
        )

    def test_coronation_day(self):
        name_1914 = "ทำบุญพระบรมอัษฐิ และพระราชพิธีฉัตรมงคล"
        name_1926 = "พระราชพิธีฉัตรมงคล"
        name_1958 = "วันฉัตรมงคล"

        for year in range(self.start_year, 1925):
            self.assertHolidayName(
                name_1914, (f"{year}-11-09", f"{year}-11-10", f"{year}-11-11", f"{year}-11-12")
            )
        self.assertNoHolidayName(name_1914, range(1926, self.end_year))
        for year in range(1926, 1936):
            self.assertHolidayName(name_1926, (f"{year}-02-24", f"{year}-02-25", f"{year}-02-26"))
        self.assertNoHolidayName(
            name_1926, range(self.start_year, 1926), range(1936, self.end_year)
        )
        self.assertHolidayName(name_1958, (f"{year}-05-05" for year in range(1958, 2017)))
        self.assertHolidayName(name_1958, (f"{year}-05-04" for year in range(2020, self.end_year)))
        self.assertNoHolidayName(name_1958, range(self.start_year, 1958), range(2017, 2020))

        self.assertNoNonObservedHoliday(
            "2012-05-07",
            "2013-05-06",
            "2024-05-06",
            "2025-05-05",
            "2030-05-06",
        )

    def test_hm_queen_suthidas_birthday(self):
        name = "วันเฉลิมพระชนมพรรษาสมเด็จพระนางเจ้าสุทิดา พัชรสุธาพิมลลักษณ พระบรมราชินี"

        self.assertHolidayName(name, (f"{year}-06-03" for year in range(2019, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2019))

        self.assertNoNonObservedHoliday(
            "2023-06-05",
            "2028-06-05",
            "2029-06-04",
        )

    def test_national_day(self):
        name_1938 = "วันขอพระราชทานรัฐธรรมนูญ"
        name_1939 = "วันชาติ"

        self.assertHolidayName(name_1938, "1938-06-24")
        for year in (*range(1940, 1948), *range(1952, 1954)):
            self.assertHolidayName(name_1939, (f"{year}-06-23", f"{year}-06-24", f"{year}-06-25"))
        self.assertHolidayName(
            name_1939,
            (f"{year}-06-24" for year in (1939, *range(1948, 1952), *range(1954, 1960))),
        )
        self.assertHolidayName(name_1939, (f"{year}-12-05" for year in range(1960, self.end_year)))
        self.assertNoHolidayName(
            name_1938, range(self.start_year, 1938), range(1939, self.end_year)
        )
        self.assertNoHolidayName(name_1939, range(self.start_year, 1939))

        # No in lieus during its existense on June 24th
        # 1960+ In lieus are same as HM King Bhumibol Adulyadej's Birthday

    def test_provisional_constitution_day(self):
        name = "วันรัฐธรรมนูญชั่วคราว"

        self.assertHolidayName(name, (f"{year}-06-27" for year in range(1938, 1940)))
        self.assertNoHolidayName(name, range(self.start_year, 1938), range(1940, self.end_year))

    def test_rama_x_birthday(self):
        name = "วันเฉลิมพระชนมพรรษาพระบาทสมเด็จพระปรเมนทรรามาธิบดีศรีสินทรมหาวชิราลงกรณ พระวชิรเกล้าเจ้าอยู่หัว"

        self.assertHolidayName(name, (f"{year}-07-28" for year in range(2017, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2017))

        self.assertNoNonObservedHoliday(
            "2018-07-30",
            "2019-07-29",
            "2024-07-29",
            "2029-07-30",
            "2030-07-29",
        )

    def test_hm_queen_sirikit_the_queen_mothers_birthday(self):
        name_1976 = "วันเฉลิมพระชนมพรรษาสมเด็จพระนางเจ้าสิริกิติ์ พระบรมราชินีนาถ"
        name_2017 = "วันเฉลิมพระชนมพรรษาสมเด็จพระบรมราชชนนีพันปีหลวง"

        self.assertHolidayName(name_1976, (f"{year}-08-12" for year in range(1976, 2017)))
        self.assertHolidayName(name_2017, (f"{year}-08-12" for year in range(2017, self.end_year)))
        self.assertNoHolidayName(
            name_1976, range(self.start_year, 1976), range(2017, self.end_year)
        )
        self.assertNoHolidayName(name_2017, range(self.start_year, 2017))

        self.assertNoNonObservedHoliday(
            "2012-08-13",
            "2017-08-14",
            "2018-08-13",
            "2023-08-14",
            "2028-08-14",
            "2029-08-13",
        )

    def test_national_mothers_day(self):
        name = "วันแม่แห่งชาติ"

        self.assertHolidayName(name, (f"{year}-04-15" for year in range(1950, 1958)))
        self.assertHolidayName(name, (f"{year}-08-12" for year in range(1976, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1950), range(1958, 1976))

        # April 15 (1950-1958) exists prior to in lieu laws
        # In lieus are same as HM Queen Sirikit's Birthday

    def test_peace_proclamation_day(self):
        name = "วันประกาศสันติภาพ"

        self.assertHolidayName(name, (f"{year}-08-16" for year in range(1946, 1948)))
        self.assertNoHolidayName(name, range(self.start_year, 1946), range(1948, self.end_year))

    def test_hm_king_bhumibol_adulyadej_memorial_day(self):
        name_2017 = "วันคล้ายวันสวรรคตพระบาทสมเด็จพระปรมินทรมหาภูมิพลอดุลยเดช บรมนาถบพิตร"
        name_2019 = "วันคล้ายวันสวรรคตพระบาทสมเด็จพระบรมชนกาธิเบศร มหาภูมิพลอดุลยเดชมหาราช บรมนาถบพิตร"
        name_2023 = "วันนวมินทรมหาราช"

        self.assertHolidayName(name_2017, (f"{year}-10-13" for year in range(2017, 2019)))
        self.assertHolidayName(name_2019, (f"{year}-10-13" for year in range(2019, 2023)))
        self.assertHolidayName(name_2023, (f"{year}-10-13" for year in range(2023, self.end_year)))
        self.assertNoHolidayName(
            name_2017, range(self.start_year, 2017), range(2019, self.end_year)
        )
        self.assertNoHolidayName(
            name_2019, range(self.start_year, 2019), range(2023, self.end_year)
        )
        self.assertNoHolidayName(name_2023, range(self.start_year, 2023))

        self.assertNoNonObservedHoliday(
            "2018-10-15",
            "2019-10-14",
            "2024-10-14",
            "2029-10-15",
            "2030-10-14",
        )

    def test_hm_king_chulalongkorn_memorial_day(self):
        name_1914 = "ทำบุญพระบรมอัษฐิพระพุทธเจ้าหลวง"
        name_1926 = "วันสวรรคตแห่งพระบาทสมเด็จพระพุทธเจ้าหลวง"
        name_1946 = "วันปิยมหาราช"

        self.assertHolidayName(
            name_1914, (f"{year}-10-23" for year in range(self.start_year, 1926))
        )
        self.assertHolidayName(name_1926, (f"{year}-10-23" for year in range(1926, 1938)))
        self.assertHolidayName(name_1946, (f"{year}-10-23" for year in range(1946, self.end_year)))
        self.assertNoHolidayName(name_1914, range(1926, self.end_year))
        self.assertNoHolidayName(
            name_1926, range(self.start_year, 1926), range(1938, self.end_year)
        )
        self.assertNoHolidayName(name_1946, range(self.start_year, 1946))

        self.assertNoNonObservedHoliday(
            "2010-10-25",
            "2011-10-24",
            "2016-10-24",
            "2021-10-25",
            "2022-10-24",
            "2027-10-25",
        )

    def test_united_nations_day(self):
        name = "วันสหประชาชาติ"
        self.assertHolidayName(name, (f"{year}-10-24" for year in range(1951, 1957)))
        self.assertNoHolidayName(name, range(self.start_year, 1951), range(1957, self.end_year))

    def test_rama_vi_to_ix_birthday(self):
        name_1914 = "เฉลิมพระชนมพรรษา"
        name_1926 = "เฉลิมพระชนม์พรรษา"
        name_1938 = "วันเฉลิมพระชนม์พรรษา"
        name_1941 = "วันเกิดในสมเด็จพระเจ้าอยู่หัว"
        name_1945 = "วันเฉลิมพระชนมพรรษา"
        name_1960 = "วันเฉลิมพระชนมพรรษาพระบาทสมเด็จพระปรมินทรมหาภูมิพลอดุลยเดช บรมนาถบพิตร"
        name_2016 = "วันคล้ายวันเฉลิมพระชนมพรรษาพระบาทสมเด็จพระปรมินทรมหาภูมิพลอดุลยเดช บรมนาถบพิตร"
        name_2019 = (
            "วันคล้ายวันเฉลิมพระชนมพรรษาพระบาทสมเด็จพระบรมชนกาธิเบศร มหาภูมิพลอดุลยเดชมหาราช บรมนาถบพิตร"
        )

        # Rama VI.
        for year in range(self.start_year, 1925):
            self.assertHolidayName(name_1914, (f"{year}-12-30", f"{year}-12-31"))
        for year in range(1915, 1926):
            self.assertHolidayName(name_1914, (f"{year}-01-01", f"{year}-01-02", f"{year}-01-03"))
        # Rama VII.
        for year in range(1926, 1935):
            self.assertHolidayName(name_1926, (f"{year}-11-07", f"{year}-11-08", f"{year}-11-09"))
        # Rama VIII.
        self.assertHolidayName(name_1938, (f"{year}-09-20" for year in range(1938, 1940)))
        self.assertHolidayName(name_1938, "1940-09-20", "1940-09-21")
        for year in range(1941, 1945):
            self.assertHolidayName(name_1941, (f"{year}-09-20", f"{year}-09-21"))
        self.assertHolidayName(name_1945, "1945-09-20", "1945-09-21")
        # Rama IX.
        for year in range(1946, 1948):
            self.assertHolidayName(name_1945, (f"{year}-12-05", f"{year}-12-06"))
        for year in (*range(1948, 1951), *range(1952, 1954)):
            self.assertHolidayName(name_1945, (f"{year}-12-04", f"{year}-12-05", f"{year}-12-06"))
        self.assertHolidayName(name_1945, "1951-12-05", "1951-12-06", "1951-12-07")
        self.assertHolidayName(name_1960, (f"{year}-12-05" for year in range(1960, 2016)))
        self.assertHolidayName(name_2016, (f"{year}-12-05" for year in range(2016, 2019)))
        self.assertHolidayName(name_2019, (f"{year}-12-05" for year in range(2019, self.end_year)))
        self.assertNoHolidayName(name_1914, range(1926, self.end_year))
        self.assertNoHolidayName(
            name_1926, range(self.start_year, 1926), range(1935, self.end_year)
        )
        self.assertNoHolidayName(
            name_1938, range(self.start_year, 1938), range(1941, self.end_year)
        )
        self.assertNoHolidayName(
            name_1941, range(self.start_year, 1941), range(1945, self.end_year)
        )
        self.assertNoHolidayName(
            name_1945, range(self.start_year, 1945), range(1960, self.end_year)
        )
        self.assertNoHolidayName(
            name_1960, range(self.start_year, 1960), range(2016, self.end_year)
        )
        self.assertNoHolidayName(
            name_2016, range(self.start_year, 2016), range(2019, self.end_year)
        )
        self.assertNoHolidayName(name_2019, range(self.start_year, 2019))

        self.assertNoNonObservedHoliday(
            "2010-12-06",
            "2015-12-07",
            "2020-12-07",
            "2021-12-06",
            "2026-12-07",
            "2027-12-06",
        )

    def test_national_fathers_day(self):
        name = "วันพ่อแห่งชาติ"

        # This concides with HM King Bhumibol Adulyadej's Birthday
        self.assertHolidayName(name, (f"{year}-12-05" for year in range(1980, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1980))

        # In lieus are same as HM King Bhumibol Adulyadej's Birthday

    def test_constitution_day(self):
        name = "วันรัฐธรรมนูญ"

        self.assertHolidayName(name, (f"{year}-12-10" for year in range(1938, self.end_year)))
        self.assertHolidayName(
            name, (f"{year}-12-09" for year in (*range(1938, 1948), *range(1950, 1954)))
        )
        self.assertHolidayName(
            name, (f"{year}-12-11" for year in (*range(1938, 1948), *range(1950, 1954)))
        )

        self.assertNoNonObservedHoliday(
            "2011-12-12",
            "2016-12-12",
            "2017-12-11",
            "2022-12-12",
            "2023-12-11",
            "2028-12-11",
        )

    def test_new_years_eve(self):
        name = "วันสิ้นปี"

        self.assertHolidayName(
            name, (f"{year}-12-31" for year in (*range(1941, 1957), *range(1989, self.end_year)))
        )
        self.assertNoHolidayName(name, range(self.start_year, 1941), range(1957, 1989))

        self.assertNoNonObservedHoliday(
            "2012-01-03",
            "2017-01-03",
            "2018-01-02",
            "2023-01-03",
            "2029-01-02",
        )

    def test_makha_bucha(self):
        name_1915 = "มาฆบูชา จาตุรงฅ์สันนิบาต"
        name_1938 = "วันมาฆบูชา"

        self.assertHolidayName(name_1915, range(1915, 1926))
        self.assertHolidayName(
            name_1938,
            "2020-02-08",
            "2021-02-26",
            "2022-02-16",
            "2023-03-06",
            "2024-02-24",
            "2025-02-12",
        )
        self.assertHolidayName(name_1938, range(1938, self.end_year))
        self.assertNoHolidayName(
            name_1915, range(self.start_year, 1915), range(1926, self.end_year)
        )
        self.assertNoHolidayName(name_1938, range(self.start_year, 1938))

        self.assertNoNonObservedHoliday(
            "2010-03-01",
            "2017-02-13",
            "2020-02-10",
            "2024-02-26",
            "2024-02-22",
            "2030-02-18",
        )

    def test_visakha_bucha(self):
        name_1914 = "วิสาขะบูชา"
        name_1926 = "วิศาขะบูชา"
        name_1938 = "วันวิสาขะบูชา"
        name_1957 = "วันวิสาขบูชา"

        self.assertHolidayName(name_1914, range(self.start_year, 1926))
        self.assertHolidayName(name_1926, range(1926, 1938))
        self.assertHolidayName(name_1938, range(1938, 1957))
        self.assertHolidayName(
            name_1957,
            "2020-05-06",
            "2021-05-26",
            "2022-05-15",
            "2023-06-03",
            "2024-05-22",
            "2025-05-11",
        )
        self.assertHolidayName(name_1957, range(1957, self.end_year))
        self.assertNoHolidayName(name_1914, range(1926, self.end_year))
        self.assertNoHolidayName(
            name_1926, range(self.start_year, 1926), range(1938, self.end_year)
        )
        self.assertNoHolidayName(
            name_1938, range(self.start_year, 1938), range(1957, self.end_year)
        )
        self.assertNoHolidayName(name_1957, range(self.start_year, 1957))

        self.assertNoNonObservedHoliday(
            "2019-05-20",
            "2022-05-16",
            "2023-06-05",
            "2025-05-12",
            "2026-06-01",
            "2029-05-28",
        )

    def test_asarnha_bucha(self):
        name = "วันอาสาฬหบูชา"

        self.assertHolidayName(
            name,
            "2020-07-05",
            "2021-07-24",
            "2022-07-13",
            "2023-08-01",
            "2024-07-20",
            "2025-07-10",
        )
        self.assertHolidayName(name, range(1962, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1962))

        self.assertNoNonObservedHoliday(
            "2017-07-10",
            "2020-07-07",
            "2021-07-26",
            "2024-07-22",
            "2027-07-20",
            "2030-07-16",
        )

    def test_buddhist_lent_day(self):
        name_1914 = "เข้าปุริมพรรษา"
        name_1938 = "วันเข้าพรรษา"

        self.assertHolidayName(name_1914, range(self.start_year, 1938))
        self.assertHolidayName(
            name_1938,
            "2020-07-06",
            "2021-07-25",
            "2022-07-14",
            "2023-08-02",
            "2024-07-21",
            "2025-07-11",
        )
        self.assertHolidayName(name_1938, range(1938, self.end_year))
        self.assertNoHolidayName(name_1914, range(1938, self.end_year))
        self.assertNoHolidayName(name_1938, range(self.start_year, 1938))

        self.assertNoNonObservedHoliday(
            "2011-07-18",
            "2014-07-14",
            "2018-07-30",
        )

    def test_royal_ploughing_ceremony(self):
        name = "วันพืชมงคล"

        self.assertNoHolidayName(name)
        self.assertGovernmentHolidayName(
            name,
            "1960-05-02",
            "1961-05-11",
            "1962-05-07",
            "1963-05-10",
            "1964-05-08",
            "1965-05-13",
            "1966-05-13",
            "1967-05-11",
            "1968-05-10",
            "1969-05-09",
            "1970-05-08",
            "1971-05-07",
            "1972-05-08",
            "1973-05-07",
            "1974-05-08",
            "1975-05-07",
            "1976-05-10",
            "1977-05-12",
            "1978-05-11",
            "1979-05-07",
            "1980-05-14",
            "1981-05-07",
            "1982-05-19",
            "1983-05-11",
            "1984-05-10",
            "1985-05-09",
            "1986-05-09",
            "1987-05-08",
            "1988-05-11",
            "1989-05-11",
            "1990-05-11",
            "1991-05-10",
            "1992-05-14",
            "1993-05-17",
            "1994-05-11",
            "1995-05-10",
            "1996-05-16",
            "1997-05-09",
            "1998-05-08",
            # Not a holiday in 1999 date, was held on MAY, 14.
            "2000-05-15",
            "2001-05-16",
            "2002-05-09",
            "2003-05-08",
            "2004-05-07",
            "2005-05-11",
            "2006-05-11",
            "2007-05-10",
            "2008-05-09",
            "2009-05-11",
            "2010-05-13",
            "2011-05-13",
            "2012-05-09",
            "2013-05-13",
            "2014-05-09",
            "2015-05-13",
            "2016-05-09",
            "2017-05-12",
            "2018-05-14",
            "2019-05-09",
            "2020-05-11",
            "2021-05-10",
            "2022-05-13",
            "2023-05-17",
            "2024-05-10",
            "2025-05-09",
            "2026-05-13",
        )
        self.assertNoGovernmentHolidayName(
            name, range(self.start_year, 1960), 1999, range(2027, self.end_year)
        )

    def test_armed_forces_day(self):
        name = "วันกองทัพไทย"

        self.assertNoHolidayName(name)
        self.assertArmedForcesHolidayName(name, (f"{year}-04-08" for year in range(1959, 1980)))
        self.assertArmedForcesHolidayName(name, (f"{year}-01-25" for year in range(1980, 2007)))
        self.assertArmedForcesHolidayName(
            name, (f"{year}-01-18" for year in range(2007, self.end_year))
        )
        self.assertNoArmedForcesHolidayName(name, range(self.start_year, 1959))

    def test_additional_closing_days_for_bank_for_agriculture_and_agircultural_cooperatives(self):
        name = "วันหยุดเพิ่มเติมสำหรับการปิดบัญชีประจำปีของธนาคารเพื่อการเกษตรและสหกรณ์การเกษตร"

        self.assertNoHolidayName(name)
        self.assertBankHolidayName(name, (f"{year}-04-01" for year in range(1943, 2022)))
        self.assertNoBankHolidayName(
            name, range(self.start_year, 1942), range(2022, self.end_year)
        )

    def test_mid_year_closing_days(self):
        name = "วันหยุดภาคครึ่งปีของสถาบันการเงินและสถาบันการเงินเฉพาะกิจ"

        self.assertNoHolidayName(name)
        self.assertBankHolidayName(name, (f"{year}-07-01" for year in range(1943, 2019)))
        self.assertNoBankHolidayName(
            name, range(self.start_year, 1942), range(2019, self.end_year)
        )

    def test_teachers_day(self):
        name = "วันครู"

        self.assertNoHolidayName(name)
        self.assertSchoolHolidayName(
            name, (f"{year}-01-16" for year in range(1957, self.end_year))
        )
        self.assertNoSchoolHolidayName(name, range(self.start_year, 1957))

    def test_thai_veterans_day(self):
        name = "วันทหารผ่านศึก"

        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-02-03" for year in range(1948, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 1948))

    def test_national_science_day(self):
        name = "วันวิทยาศาสตร์แห่งชาติ"

        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-08-18" for year in range(1982, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 1982))

    def test_national_artist_day(self):
        name = "วันศิลปินแห่งชาติ"

        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-02-26" for year in range(1985, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 1985))

    def test_international_womens_day(self):
        name = "วันสตรีสากล"

        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-03-08" for year in range(1989, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 1989))

    def test_national_forest_conservation_day(self):
        name = "วันอนุรักษ์ทรัพยากรป่าไม้ของชาติ"

        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-01-14" for year in range(1990, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 1990))

    def test_hm_king_ramkamhaeng_memorial_day(self):
        name = "วันพ่อขุนรามคำแหงมหาราช"

        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-01-17" for year in range(1990, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 1990))

    def test_national_aviation_day(self):
        name = "วันการบินแห่งชาติ"

        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-01-13" for year in range(1995, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 1995))

    def test_thai_national_flag_day(self):
        name = "วันพระราชทานธงชาติไทย"

        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-09-28" for year in range(2017, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2017))

    def test_loy_krathong(self):
        name = "วันลอยกระทง"

        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name,
            "2020-10-31",
            "2021-11-19",
            "2022-11-08",
            "2023-11-27",
            "2024-11-15",
            "2025-11-05",
        )
        self.assertWorkdayHolidayName(name, self.full_range)

    def test_weekends(self):
        for dt in (
            "1940-01-07",  # SUN.
            "1940-01-14",  # SUN.
            "1940-01-21",  # SUN.
            "1940-01-28",  # SUN.
            "1956-09-02",  # SUN.
            "1956-09-09",  # SUN.
            "1956-09-16",  # SUN.
            "1956-09-23",  # SUN.
            "1956-09-30",  # SUN.
            "1956-10-04",  # THU (Buddhist Sabbath Day).
            "1956-10-07",  # SUN.
            "1956-10-12",  # FRI (Buddhist Sabbath Day).
            "1956-10-14",  # SUN.
            "1956-10-19",  # FRI (Buddhist Sabbath Day).
            "1956-10-21",  # SUN.
            "1956-10-27",  # SAT (Buddhist Sabbath Day).
            "1956-10-28",  # SUN.
            "1957-01-06",  # SUN.
            "1957-01-08",  # TUE (Buddhist Sabbath Day).
            "1957-01-13",  # SUN.
            "1957-01-15",  # TUE (Buddhist Sabbath Day).
            "1957-01-20",  # SUN.
            "1957-01-23",  # WED (Buddhist Sabbath Day).
            "1957-01-27",  # SUN.
            "1957-01-30",  # WED (Buddhist Sabbath Day).
            "1957-09-01",  # SUN.
            "1957-09-02",  # MON (Buddhist Sabbath Day).
            "1957-09-08",  # SUN.
            "1957-09-09",  # MON (Buddhist Sabbath Day).
            "1957-09-15",  # SUN.
            "1957-09-17",  # TUE (Buddhist Sabbath Day).
            "1957-09-22",  # SUN.
            "1957-09-24",  # TUE (Buddhist Sabbath Day).
            "1957-09-29",  # SUN.
            "1957-10-02",  # WED (Buddhist Sabbath Day).
            "1957-10-06",  # SUN.
            "1957-10-13",  # SUN.
            "1957-10-20",  # SUN.
            "1957-10-27",  # SUN.
            "1958-01-05",  # SUN.
            "1958-01-12",  # SUN.
            "1958-01-19",  # SUN.
            "1958-01-26",  # SUN.
            "1960-01-02",  # SAT.
            "1960-01-03",  # SUN.
            "1960-01-09",  # SAT.
            "1960-01-10",  # SUN.
            "1960-01-16",  # SAT.
            "1960-01-17",  # SUN.
            "1960-01-23",  # SAT.
            "1960-01-24",  # SUN.
            "1960-01-30",  # SAT.
            "1960-01-31",  # SUN.
        ):
            self.assertTrue(self.holidays.is_weekend(dt))

        for dt in (
            "1938-01-01",  # SAT.
            "1938-01-02",  # SUN.
            "1938-01-08",  # SAT.
            "1938-01-09",  # SUN.
            "1938-01-15",  # SAT.
            "1938-01-16",  # SUN.
            "1938-01-22",  # SAT.
            "1938-01-23",  # SUN.
            "1938-01-29",  # SAT.
            "1938-01-30",  # SUN.
            "1940-01-06",  # SAT (Half Day).
            "1940-01-13",  # SAT (Half Day).
            "1940-01-20",  # SAT (Half Day).
            "1940-01-27",  # SAT (Half Day).
            "1956-09-01",  # SAT (Half Day).
            "1956-09-08",  # SAT (Half Day).
            "1956-09-15",  # SAT (Half Day).
            "1956-09-22",  # SAT (Half Day).
            "1956-09-29",  # SAT (Half Day).
            "1956-10-06",  # SAT.
            "1956-10-13",  # SAT.
            "1956-10-20",  # SAT.
            "1957-01-05",  # SAT.
            "1957-01-12",  # SAT.
            "1957-01-19",  # SAT.
            "1957-01-26",  # SAT.
            "1957-09-07",  # SAT.
            "1957-09-14",  # SAT.
            "1957-09-21",  # SAT.
            "1957-09-28",  # SAT.
            "1957-10-05",  # SAT.
            "1957-10-12",  # SAT (Half Day).
            "1957-10-19",  # SAT (Half Day).
            "1957-10-26",  # SAT (Half Day).
            "1958-01-04",  # SAT (Half Day).
            "1958-01-11",  # SAT (Half Day).
            "1958-01-18",  # SAT (Half Day).
            "1958-01-25",  # SAT (Half Day).
        ):
            self.assertFalse(self.holidays.is_weekend(dt))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "วันขึ้นปีใหม่"),
            ("2022-01-03", "ชดเชยวันขึ้นปีใหม่"),
            ("2022-01-08", "วันเด็กแห่งชาติ"),
            ("2022-01-13", "วันการบินแห่งชาติ"),
            ("2022-01-14", "วันอนุรักษ์ทรัพยากรป่าไม้ของชาติ"),
            ("2022-01-16", "วันครู"),
            ("2022-01-17", "วันพ่อขุนรามคำแหงมหาราช"),
            ("2022-01-18", "วันกองทัพไทย"),
            ("2022-02-03", "วันทหารผ่านศึก"),
            ("2022-02-16", "วันมาฆบูชา"),
            ("2022-02-26", "วันศิลปินแห่งชาติ"),
            ("2022-03-08", "วันสตรีสากล"),
            ("2022-04-06", "วันพระบาทสมเด็จพระพุทธยอดฟ้าจุฬาโลกมหาราช และวันที่ระลึกมหาจักรีบรมราชวงศ์"),
            ("2022-04-13", "วันสงกรานต์"),
            ("2022-04-14", "วันสงกรานต์"),
            ("2022-04-15", "วันสงกรานต์"),
            ("2022-05-01", "วันแรงงานแห่งชาติ"),
            ("2022-05-02", "ชดเชยวันแรงงานแห่งชาติ"),
            ("2022-05-04", "วันฉัตรมงคล"),
            ("2022-05-13", "วันพืชมงคล"),
            ("2022-05-15", "วันวิสาขบูชา"),
            ("2022-05-16", "ชดเชยวันวิสาขบูชา"),
            (
                "2022-06-03",
                "วันเฉลิมพระชนมพรรษาสมเด็จพระนางเจ้าสุทิดา พัชรสุธาพิมลลักษณ พระบรมราชินี",
            ),
            ("2022-07-13", "วันอาสาฬหบูชา"),
            ("2022-07-14", "วันเข้าพรรษา"),
            ("2022-07-15", "วันหยุดพิเศษ (เพิ่มเติม)"),
            (
                "2022-07-28",
                "วันเฉลิมพระชนมพรรษาพระบาทสมเด็จพระปรเมนทรรามาธิบดี"
                "ศรีสินทรมหาวชิราลงกรณ พระวชิรเกล้าเจ้าอยู่หัว",
            ),
            ("2022-07-29", "วันหยุดพิเศษ (เพิ่มเติม)"),
            ("2022-08-12", "วันเฉลิมพระชนมพรรษาสมเด็จพระบรมราชชนนีพันปีหลวง; วันแม่แห่งชาติ"),
            ("2022-08-18", "วันวิทยาศาสตร์แห่งชาติ"),
            ("2022-09-28", "วันพระราชทานธงชาติไทย"),
            (
                "2022-10-13",
                "วันคล้ายวันสวรรคตพระบาทสมเด็จพระบรมชนกาธิเบศร มหาภูมิพลอดุลยเดชมหาราช บรมนาถบพิตร",
            ),
            ("2022-10-14", "วันหยุดพิเศษ (เพิ่มเติม)"),
            ("2022-10-23", "วันปิยมหาราช"),
            ("2022-10-24", "ชดเชยวันปิยมหาราช"),
            ("2022-11-08", "วันลอยกระทง"),
            (
                "2022-12-05",
                "วันคล้ายวันเฉลิมพระชนมพรรษาพระบาทสมเด็จพระบรม"
                "ชนกาธิเบศร มหาภูมิพลอดุลยเดชมหาราช บรมนาถบพิตร; วันชาติ; วันพ่อแห่งชาติ",
            ),
            ("2022-12-10", "วันรัฐธรรมนูญ"),
            ("2022-12-12", "ชดเชยวันรัฐธรรมนูญ"),
            ("2022-12-30", "วันหยุดพิเศษ (เพิ่มเติม)"),
            ("2022-12-31", "วันสิ้นปี"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "New Year's Day (in lieu)"),
            ("2022-01-08", "National Children's Day"),
            ("2022-01-13", "National Aviation Day"),
            ("2022-01-14", "National Forest Conservation Day"),
            ("2022-01-16", "Teacher's Day"),
            ("2022-01-17", "HM King Ramkhamhaeng Memorial Day"),
            ("2022-01-18", "Royal Thai Armed Forces Day"),
            ("2022-02-03", "Thai Veterans Day"),
            ("2022-02-16", "Makha Bucha"),
            ("2022-02-26", "National Artist Day"),
            ("2022-03-08", "International Women's Day"),
            ("2022-04-06", "Chakri Memorial Day"),
            ("2022-04-13", "Songkran Festival"),
            ("2022-04-14", "Songkran Festival"),
            ("2022-04-15", "Songkran Festival"),
            ("2022-05-01", "National Labor Day"),
            ("2022-05-02", "National Labor Day (in lieu)"),
            ("2022-05-04", "Coronation Day"),
            ("2022-05-13", "Royal Ploughing Ceremony"),
            ("2022-05-15", "Visakha Bucha"),
            ("2022-05-16", "Visakha Bucha (in lieu)"),
            ("2022-06-03", "HM Queen Suthida's Birthday"),
            ("2022-07-13", "Asarnha Bucha"),
            ("2022-07-14", "Buddhist Lent Day"),
            ("2022-07-15", "Bridge Public Holiday"),
            ("2022-07-28", "HM King Maha Vajiralongkorn's Birthday"),
            ("2022-07-29", "Bridge Public Holiday"),
            (
                "2022-08-12",
                "HM Queen Sirikit The Queen Mother's Birthday; National Mother's Day",
            ),
            ("2022-08-18", "National Science Day"),
            ("2022-09-28", "Thai National Flag Day"),
            ("2022-10-13", "HM King Bhumibol Adulyadej the Great Memorial Day"),
            ("2022-10-14", "Bridge Public Holiday"),
            ("2022-10-23", "HM King Chulalongkorn Memorial Day"),
            ("2022-10-24", "HM King Chulalongkorn Memorial Day (in lieu)"),
            ("2022-11-08", "Loy Krathong"),
            (
                "2022-12-05",
                "HM King Bhumibol Adulyadej the Great's Birthday; National Day; "
                "National Father's Day",
            ),
            ("2022-12-10", "Constitution Day"),
            ("2022-12-12", "Constitution Day (in lieu)"),
            ("2022-12-30", "Bridge Public Holiday"),
            ("2022-12-31", "New Year's Eve"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-01-03", "Новий рік (вихідний)"),
            ("2022-01-08", "Національний день дітей"),
            ("2022-01-13", "Національний день авіації"),
            ("2022-01-14", "Національний день охорони лісів"),
            ("2022-01-16", "День вчителя"),
            ("2022-01-17", "День памʼяті короля Рамкхамхенга"),
            ("2022-01-18", "День Королівських збройних сил Таїланду"),
            ("2022-02-03", "День ветеранів"),
            ("2022-02-16", "Маха Буча"),
            ("2022-02-26", "Національний день художника"),
            ("2022-03-08", "Міжнародний жіночий день"),
            ("2022-04-06", "День памʼяті короля Рами I та династії Чакрі"),
            ("2022-04-13", "Тайський Новий рік"),
            ("2022-04-14", "Тайський Новий рік"),
            ("2022-04-15", "Тайський Новий рік"),
            ("2022-05-01", "Національний день праці"),
            ("2022-05-02", "Національний день праці (вихідний)"),
            ("2022-05-04", "День коронації"),
            ("2022-05-13", "Церемонія королівської оранки"),
            ("2022-05-15", "Вісака Буча"),
            ("2022-05-16", "Вісака Буча (вихідний)"),
            ("2022-06-03", "День народження Її Величності королеви Сутіди"),
            ("2022-07-13", "Асарна Буча"),
            ("2022-07-14", "Початок буддистського посту"),
            ("2022-07-15", "Проміжний вихідний"),
            ("2022-07-28", "День народження Його Величності короля Маха Вачіралонгкорна"),
            ("2022-07-29", "Проміжний вихідний"),
            (
                "2022-08-12",
                "День народження Її Величності королеви-матері Сірікіт; Національний день матері",
            ),
            ("2022-08-18", "Національний день науки"),
            ("2022-09-28", "День національного прапора Таїланду"),
            ("2022-10-13", "Річниця смерті Його Величності короля Пуміпона Адульядета Великого"),
            ("2022-10-14", "Проміжний вихідний"),
            ("2022-10-23", "День памʼяті Його Величності короля Чулалонгкорна"),
            ("2022-10-24", "День памʼяті Його Величності короля Чулалонгкорна (вихідний)"),
            ("2022-11-08", "Лой Кратонг"),
            (
                "2022-12-05",
                "Національний день; Національний день батька; "
                "Річниця дня народження Його Величності короля Пуміпона Адульядета Великого",
            ),
            ("2022-12-10", "День Конституції"),
            ("2022-12-12", "День Конституції (вихідний)"),
            ("2022-12-30", "Проміжний вихідний"),
            ("2022-12-31", "Переддень Нового року"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.latvia import Latvia
from tests.common import CommonCountryTests, WorkingDayTests


class TestLatvia(CommonCountryTests, WorkingDayTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Latvia)

    def test_special_holidays(self):
        self.assertHoliday(
            "2018-07-09",
            "2018-09-24",
            "2023-05-29",
            "2023-07-10",
        )

    def test_substituted_holidays(self):
        self.assertHoliday(
            "2001-12-24",
            "2002-12-30",
            "2003-11-17",
            "2004-01-02",
            "2004-05-03",
            "2004-06-25",
            "2004-11-19",
            "2006-11-27",
            "2007-04-30",
            "2008-05-02",
            "2008-11-17",
            "2009-01-02",
            "2009-06-22",
            "2010-05-03",
            "2010-06-25",
            "2010-11-19",
            "2012-04-30",
            "2013-12-23",
            "2013-12-30",
            "2014-05-02",
            "2014-11-17",
            "2015-01-02",
            "2015-06-22",
            "2017-05-05",
            "2018-04-30",
            "2020-06-22",
            "2021-05-03",
            "2021-06-25",
            "2021-11-19",
            "2023-05-05",
            "2024-12-23",
            "2024-12-30",
            "2025-05-02",
            "2025-11-17",
            "2026-01-02",
            "2026-06-22",
        )

    def test_workdays(self):
        self.assertWorkingDay(
            "2001-12-22",
            "2002-12-28",
            "2003-11-15",
            "2004-01-10",
            "2004-05-08",
            "2004-06-19",
            "2004-11-13",
            "2006-12-02",
            "2007-04-14",
            "2008-05-10",
            "2008-11-22",
            "2009-01-10",
            "2009-06-27",
            "2010-05-29",
            "2010-06-19",
            "2010-11-13",
            "2012-04-28",
            "2013-12-14",
            "2013-12-28",
            "2014-05-10",
            "2014-11-22",
            "2015-01-10",
            "2015-06-27",
            "2017-05-13",
            "2018-04-21",
            "2020-06-13",
            "2021-05-08",
            "2021-06-19",
            "2021-11-13",
            "2023-05-20",
            "2024-12-14",
            "2024-12-28",
            "2025-05-10",
            "2025-11-08",
            "2026-01-17",
            "2026-06-27",
        )

    def test_new_years(self):
        self.assertHolidayName("Jaungada diena", (f"{year}-01-01" for year in self.full_range))

    def test_good_friday(self):
        name = "Lielā Piektdiena"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_sunday(self):
        name = "Pirmās Lieldienas"
        self.assertHolidayName(
            name,
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Otrās Lieldienas"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, range(1997, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1997))

    def test_labor_day(self):
        self.assertHolidayName("Darba svētki", (f"{year}-05-01" for year in self.full_range))

    def test_constitutional_assembly_convocation_day(self):
        self.assertHolidayName(
            "Latvijas Republikas Satversmes sapulces sasaukšanas diena",
            (f"{year}-05-01" for year in self.full_range),
        )

    def test_restoration_of_independence_day(self):
        name_2002 = "Latvijas Republikas Neatkarības deklarācijas pasludināšanas diena"
        name_2012 = "Latvijas Republikas Neatkarības atjaunošanas diena"
        name_2002_observed = f"{name_2002} (brīvdiena)"
        name_2012_observed = f"{name_2012} (brīvdiena)"
        self.assertHolidayName(name_2002, (f"{year}-05-04" for year in range(2002, 2012)))
        self.assertHolidayName(name_2012, (f"{year}-05-04" for year in range(2012, self.end_year)))
        self.assertNoHolidayName(
            name_2002, range(self.start_year, 2002), range(2012, self.end_year)
        )
        self.assertNoHolidayName(name_2012, range(self.start_year, 2012))

        obs_dts_1 = ("2008-05-05",)
        obs_dts_2 = (
            "2013-05-06",
            "2014-05-05",
            "2019-05-06",
        )

        self.assertHolidayName(name_2002_observed, obs_dts_1)
        self.assertHolidayName(name_2012_observed, obs_dts_2)
        self.assertNoNonObservedHoliday(obs_dts_1, obs_dts_2)
        self.assertNoHolidayName(name_2002_observed, range(self.start_year, 2008))
        self.assertNoHolidayName(name_2012_observed, range(self.start_year, 2008))

    def test_mothers_day(self):
        name = "Mātes diena"
        self.assertHolidayName(
            name,
            "2020-05-10",
            "2021-05-09",
            "2022-05-08",
            "2023-05-14",
            "2024-05-12",
            "2025-05-11",
        )
        self.assertHolidayName(name, self.full_range)

    def test_whit_sunday(self):
        name = "Vasarsvētki"
        self.assertHolidayName(
            name,
            "2020-05-31",
            "2021-05-23",
            "2022-06-05",
            "2023-05-28",
            "2024-05-19",
            "2025-06-08",
        )
        self.assertHolidayName(name, range(1996, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1996))

    def test_midsummer_eve(self):
        self.assertHolidayName("Līgo diena", (f"{year}-06-23" for year in self.full_range))

    def test_midsummer_day(self):
        self.assertHolidayName("Jāņu diena", (f"{year}-06-24" for year in self.full_range))

    def test_republic_of_latvia_proclamation_day(self):
        name = "Latvijas Republikas Proklamēšanas diena"
        name_observed = f"{name} (brīvdiena)"
        self.assertHolidayName(name, (f"{year}-11-18" for year in self.full_range))

        obs_dts = (
            "2007-11-19",
            "2012-11-19",
            "2017-11-20",
            "2018-11-19",
            "2023-11-20",
        )
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNoHolidayName(name_observed, range(self.start_year, 2007))

    def test_christmas_eve(self):
        name = "Ziemassvētku vakars"
        self.assertHolidayName(name, (f"{year}-12-24" for year in range(2007, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2007))

    def test_christmas_day(self):
        self.assertHolidayName(
            "Pirmie Ziemassvētki", (f"{year}-12-25" for year in self.full_range)
        )

    def test_second_day_of_christmas(self):
        self.assertHolidayName("Otrie Ziemassvētki", (f"{year}-12-26" for year in self.full_range))

    def test_new_years_eve(self):
        self.assertHolidayName("Vecgada diena", (f"{year}-12-31" for year in self.full_range))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Jaungada diena"),
            ("2022-04-15", "Lielā Piektdiena"),
            ("2022-04-17", "Pirmās Lieldienas"),
            ("2022-04-18", "Otrās Lieldienas"),
            (
                "2022-05-01",
                "Darba svētki; Latvijas Republikas Satversmes sapulces sasaukšanas diena",
            ),
            ("2022-05-04", "Latvijas Republikas Neatkarības atjaunošanas diena"),
            ("2022-05-08", "Mātes diena"),
            ("2022-06-05", "Vasarsvētki"),
            ("2022-06-23", "Līgo diena"),
            ("2022-06-24", "Jāņu diena"),
            ("2022-11-18", "Latvijas Republikas Proklamēšanas diena"),
            ("2022-12-24", "Ziemassvētku vakars"),
            ("2022-12-25", "Pirmie Ziemassvētki"),
            ("2022-12-26", "Otrie Ziemassvētki"),
            ("2022-12-31", "Vecgada diena"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Constitutional Assembly Convocation Day; Labor Day"),
            ("2022-05-04", "Restoration of Independence Day"),
            ("2022-05-08", "Mother's Day"),
            ("2022-06-05", "Whit Sunday"),
            ("2022-06-23", "Midsummer Eve"),
            ("2022-06-24", "Midsummer Day"),
            ("2022-11-18", "Republic of Latvia Proclamation Day"),
            ("2022-12-24", "Christmas Eve"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Second Day of Christmas"),
            ("2022-12-31", "New Year's Eve"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-17", "Великдень"),
            ("2022-04-18", "Великодній понеділок"),
            (
                "2022-05-01",
                "День праці; День скликання Конституційних зборів Латвійської Республіки",
            ),
            ("2022-05-04", "День відновлення незалежности Латвійської Республіки"),
            ("2022-05-08", "День матері"),
            ("2022-06-05", "Трійця"),
            ("2022-06-23", "Ліго"),
            ("2022-06-24", "Янів день"),
            ("2022-11-18", "День проголошення Латвійської Республіки"),
            ("2022-12-24", "Святий вечір"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "Другий день Різдва"),
            ("2022-12-31", "Переддень Нового року"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.montserrat import Montserrat
from tests.common import CommonCountryTests


class TestMontserrat(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(2017, 2050)
        super().setUpClass(Montserrat, years=years, years_non_observed=years)

    def test_special_holidays(self):
        self.assertHolidayName(
            "Special Public Holiday",
            "2018-09-14",
            "2019-05-10",
            "2020-07-15",
            "2023-04-12",
        )

        for dt, name in (
            ("2022-06-03", "Platinum Jubilee of Elizabeth II"),
            ("2022-09-19", "National Day of Mourning"),
            ("2023-05-06", "Coronation of King Charles III"),
        ):
            self.assertHolidayName(name, dt)

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in range(2017, 2050)))
        obs_dt = (
            "2017-01-03",
            "2018-01-02",
            "2022-01-03",
            "2023-01-03",
            "2024-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_saint_patricks_day(self):
        name = "Saint Patrick's Day"
        self.assertHolidayName(name, (f"{year}-03-17" for year in range(2017, 2050)))
        obs_dt = (
            "2018-03-19",
            "2019-03-18",
            "2024-03-18",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_good_friday(self):
        name = "Good Friday"
        dts = (
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, range(2017, 2050))

    def test_easter_monday(self):
        name = "Easter Monday"
        dts = (
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, range(2017, 2050))

    def test_labor_day(self):
        name = "Labour Day"
        dts = (
            "2022-05-02",
            "2023-05-01",
            "2024-05-06",
            "2025-05-05",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, range(2017, 2050))

    def test_queens_birthday(self):
        name = "Queen's Birthday"
        dts = (
            "2019-06-10",
            "2020-06-15",
            "2021-06-14",
            "2022-06-02",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, range(2017, 2023))
        self.assertNoHolidayName(name, range(2023, 2050))

    def test_kings_birthday(self):
        name = "King's Birthday"
        dts = (
            "2023-06-19",
            "2024-06-17",
            "2025-06-16",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, range(2023, 2050))
        self.assertNoHolidayName(name, range(2017, 2023))

    def test_whit_monday(self):
        name = "Whit Monday"
        dts = (
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertHolidayName(name, dts)
        obs_dt = ("2019-06-11",)
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_day_of_prayer_and_thanksgiving(self):
        first_name = "National Day of Prayer and Thanksgiving"
        self.assertHolidayName(
            first_name,
            "2021-07-21",
            "2022-07-20",
            "2023-07-12",
        )
        self.assertNoHolidayName(first_name, range(2017, 2021), range(2024, 2050))
        second_name = "Day of Prayer and Thanksgiving"
        self.assertHolidayName(
            second_name,
            "2024-07-10",
            "2025-07-09",
        )
        self.assertHolidayName(second_name, range(2024, 2050))
        self.assertNoHolidayName(second_name, range(2017, 2024))

    def test_emancipation_day(self):
        name = "Emancipation Day"
        dts = (
            "2022-08-01",
            "2023-08-07",
            "2024-08-05",
            "2025-08-04",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, range(2017, 2050))

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in range(2017, 2050)))
        obs_dt = (
            "2021-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in range(2017, 2050)))
        obs_dt = (
            "2020-12-28",
            "2021-12-28",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_festival_day(self):
        name = "Festival Day"
        self.assertHolidayName(name, (f"{year}-12-31" for year in range(2017, 2050)))
        obs_dt = (
            "2017-01-02",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "New Year's Day"),
            ("2024-01-02", "New Year's Day (observed)"),
            ("2024-03-17", "Saint Patrick's Day"),
            ("2024-03-18", "Saint Patrick's Day (observed)"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-05-06", "Labour Day"),
            ("2024-05-20", "Whit Monday"),
            ("2024-06-17", "King's Birthday"),
            ("2024-07-10", "Day of Prayer and Thanksgiving"),
            ("2024-08-05", "Emancipation Day"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
            ("2024-12-31", "Festival Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-01-02", "New Year's Day (observed)"),
            ("2024-03-17", "Saint Patrick's Day"),
            ("2024-03-18", "Saint Patrick's Day (observed)"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-05-06", "Labor Day"),
            ("2024-05-20", "Whit Monday"),
            ("2024-06-17", "King's Birthday"),
            ("2024-07-10", "Day of Prayer and Thanksgiving"),
            ("2024-08-05", "Emancipation Day"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
            ("2024-12-31", "Festival Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import unittest

from holidays.calendars.ethiopian import is_ethiopian_leap_year


class TestEthiopianCalendar(unittest.TestCase):
    def test_is_ethiopian_leap_year(self):
        known_ethiopian_leap_years = {
            # Known Cases.
            2022: False,
            2023: True,
            2024: False,
            2025: False,
            # Future Cases.
            2098: False,
            2099: True,
            2100: False,
            2101: False,
            2198: False,
            2199: True,
            2200: False,
            2201: False,
        }
        for year in known_ethiopian_leap_years:
            self.assertEqual(known_ethiopian_leap_years[year], is_ethiopian_leap_year(year))



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import MAY, JUN, JUL, AUG, SEP, OCT, NOV
from holidays.constants import PUBLIC, WORKDAY
from holidays.groups import ChristianHolidays, InternationalHolidays, IslamicHolidays
from holidays.holiday_base import HolidayBase


class Togo(HolidayBase, ChristianHolidays, InternationalHolidays, IslamicHolidays):
    """Togo holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Togo>
        * <https://web.archive.org/web/20250507123116/https://www.timeanddate.com/holidays/togo/>
        * <https://web.archive.org/web/20250507123425/https://www.goethe.de/ins/tg/fr/ueb/fer.html>
        * <https://web.archive.org/web/20250507123619/https://www.republiquetogolaise.com/politique/2309-3579-23-septembre-le-togo-rend-hommage-a-ses-martyrs>
        * <https://web.archive.org/web/20250507123840/https://islam.zmo.de/s/westafrica/item/25841#?xywh=-405,-94,2376,1868>
        * <https://en.wikipedia.org/wiki/1986_Togolese_coup_attempt>
        * <https://web.archive.org/web/20250507131725/https://www.rfi.fr/fr/afrique/20140112-togo-le-13-janvier-est-plus-jour-fete>
        * <https://web.archive.org/web/20250507124528/https://islam.zmo.de/s/afrique_ouest/item/25800#?xywh=-1641,0,4303,2363>
        * <https://web.archive.org/web/20231202052731/https://www.holidayscalendar.com/event/anniversary-of-the-failed-attack-on-lome/>

    Ramadan start dates:
        * [2015](https://web.archive.org/web/20150825071241/https://www.republicoftogo.com/toutes-les-rubriques/societe/le-mois-du-jeune-debute-le-18-juin)
        * [2016](https://web.archive.org/web/20250507125256/https://www.republicoftogo.com/toutes-les-rubriques/societe/le-ramadan-debute-le-6-juin)
        * [2017](https://web.archive.org/web/20250507125353/https://www.tf1info.fr/societe/le-ramadan-2017-1438-commence-le-samedi-27-mai-comment-la-date-du-debut-du-jeune-est-elle-fixee-1512235.html)
        * [2018](https://web.archive.org/web/20250507125622/https://www.republicoftogo.com/toutes-les-rubriques/societe/debut-du-ramadan-demain)
        * [2019](https://web.archive.org/web/20250507130404/https://www.republiquetogolaise.com/social/0605-3100-la-communaute-musulmane-du-togo-entame-ce-lundi-le-jeune-du-mois-de-ramadan)
        * [2020](https://web.archive.org/web/20250507125837/https://www.republiquetogolaise.com/culture/2404-4278-debut-du-mois-de-ramadan)
        * [2021](https://web.archive.org/web/20250507130643/https://www.republiquetogolaise.com/culture/1304-5393-debut-du-mois-de-ramadan)
        * [2022](https://web.archive.org/web/20250507130847/https://www.republiquetogolaise.com/culture/0304-6745-debut-du-mois-de-ramadan)
        * [2023](https://web.archive.org/web/20250507131022/https://www.republiquetogolaise.com/culture/2303-7864-debut-du-mois-de-ramadan)
        * [2024](https://web.archive.org/web/20250507131215/https://www.republiquetogolaise.com/culture/1103-9017-debut-du-mois-de-ramadan)
        * [2025](https://web.archive.org/web/20250507131439/https://www.republiquetogolaise.com/culture/2802-10360-le-jeune-de-ramadan-debute-le-1er-mars)
    """

    country = "TG"
    default_language = "fr"
    supported_categories = (PUBLIC, WORKDAY)
    # %s (estimated).
    estimated_label = tr("%s (estimé)")
    supported_languages = ("en_US", "fr")
    # Togo gained independence on April 27, 1960.
    start_year = 1961

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=TogoIslamicHolidays, show_estimated=islamic_show_estimated
        )
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Jour de l'an"))

        if 1967 <= self._year <= 2013:
            # Liberation Day.
            self._add_holiday_jan_13(tr("Fête de la libération nationale"))

        # Easter Monday.
        self._add_easter_monday(tr("Lundi de Pâques"))

        # Independence Day.
        self._add_holiday_apr_27(tr("Fête de l'indépendance"))

        # Labor Day.
        self._add_holiday_may_1(tr("Fête du travail"))

        # Ascension Day.
        self._add_ascension_thursday(tr("Fête de l'Ascension"))

        # Whit Monday.
        self._add_whit_monday(tr("Lundi de Pentecôte"))

        # Martyrs' Day.
        self._add_holiday_jun_21(tr("Fête des Martyrs"))

        # Assumption Day.
        self._add_holiday_aug_15(tr("Assomption"))

        # All Saints' Day.
        self._add_all_saints_day(tr("Toussaint"))

        # Christmas Day.
        self._add_christmas_day(tr("Noël"))

        # First Day of Ramadan.
        self._add_ramadan_beginning_day(tr("Ramadan"))

        # Eid al-Fitr.
        self._add_eid_al_fitr_day(tr("l'Aïd El-Fitr"))

        # Eid al-Adha.
        self._add_eid_al_adha_day(tr("Tabaski"))

    def _populate_workday_holidays(self):
        if self._year >= 1987:
            # Anniversary of the Failed Attack on Lomé.
            self._add_holiday_sep_24(tr("Anniversaire de l'attentat manqué contre Lomé"))


class TG(Togo):
    pass


class TGO(Togo):
    pass


class TogoIslamicHolidays(_CustomIslamicHolidays):
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2010, 2025)
    EID_AL_ADHA_DATES = {
        2010: (NOV, 17),
        2011: (NOV, 7),
        2014: (OCT, 5),
        2015: (SEP, 24),
        2016: (SEP, 13),
        2017: (SEP, 2),
        2018: (AUG, 22),
        2025: (JUN, 7),
    }

    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2010, 2025)
    EID_AL_FITR_DATES = {
        2011: (AUG, 31),
        2014: (JUL, 29),
        2015: (JUL, 18),
        2016: (JUL, 7),
        2017: (JUN, 26),
        2019: (JUN, 5),
    }

    RAMADAN_BEGINNING_DATES_CONFIRMED_YEARS = (2015, 2025)
    RAMADAN_BEGINNING_DATES = {
        2018: (MAY, 17),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.cameroon import Cameroon
from tests.common import CommonCountryTests


class TestCameroon(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Cameroon)

    def test_special_holidays(self):
        self.assertHoliday("2021-05-14", "2021-07-19")

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2012-01-02",
            "2017-01-02",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_youth_day(self):
        name = "Youth Day"
        self.assertHolidayName(name, (f"{year}-02-11" for year in range(1966, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1966))
        obs_dts = (
            "2018-02-12",
            "2024-02-12",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_labor_day(self):
        name = "Labour Day"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dts = (
            "2016-05-02",
            "2022-05-03",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_national_day(self):
        name = "National Day"
        self.assertHolidayName(name, (f"{year}-05-20" for year in range(1972, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1972))
        obs_dts = (
            "2012-05-21",
            "2018-05-21",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_assumption_day(self):
        name = "Assumption Day"
        self.assertHolidayName(name, (f"{year}-08-15" for year in self.full_range))
        obs_dts = (
            "2010-08-16",
            "2021-08-16",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2011-12-26",
            "2016-12-26",
            "2022-12-26",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_fitr(self):
        name = "Eid al-Fitr"
        self.assertHolidayName(
            name,
            "2018-06-15",
            "2019-06-04",
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2012-08-20",
            "2020-05-25",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_adha(self):
        name = "Eid al-Adha"
        self.assertHolidayName(
            name,
            "2018-08-21",
            "2019-08-11",
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2007-01-02",  # special case
            "2014-10-06",
            "2019-08-12",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_prophets_birthday(self):
        name = "Mawlid"
        self.assertHolidayName(
            name,
            "2018-11-21",
            "2019-11-10",
            "2020-10-29",
            "2021-10-19",
            "2022-10-08",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2012-02-06",
            "2019-11-11",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "New Year's Day"),
            ("2022-02-11", "Youth Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-05-01", "Labour Day"),
            ("2022-05-02", "Eid al-Fitr"),
            ("2022-05-03", "Labour Day (observed)"),
            ("2022-05-20", "National Day"),
            ("2022-05-26", "Ascension Day"),
            ("2022-07-09", "Eid al-Adha"),
            ("2022-08-15", "Assumption Day"),
            ("2022-10-08", "Mawlid"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Christmas Day (observed)"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.liechtenstein import Liechtenstein
from tests.common import CommonCountryTests


class TestLiechtenstein(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Liechtenstein)

    def test_new_years_day(self):
        self.assertHolidayName("Neujahr", (f"{year}-01-01" for year in self.full_range))

    def test_saint_berchtolds_day(self):
        name = "Berchtoldstag"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(name, (f"{year}-01-02" for year in self.full_range))

    def test_epiphany(self):
        self.assertHolidayName(
            "Heilige Drei Könige", (f"{year}-01-06" for year in self.full_range)
        )

    def test_candlemas(self):
        self.assertHolidayName("Mariä Lichtmess", (f"{year}-02-02" for year in self.full_range))

    def test_shrove_tuesday(self):
        name = "Fasnachtsdienstag"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(
            name,
            "2020-02-25",
            "2021-02-16",
            "2022-03-01",
            "2023-02-21",
            "2024-02-13",
            "2025-03-04",
        )
        self.assertBankHolidayName(name, self.full_range)

    def test_saint_josephs_day(self):
        self.assertHolidayName("Josefstag", (f"{year}-03-19" for year in self.full_range))

    def test_good_friday(self):
        name = "Karfreitag"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertBankHolidayName(name, self.full_range)

    def test_easter_sunday(self):
        name = "Ostersonntag"
        self.assertHolidayName(
            name,
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Ostermontag"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        self.assertHolidayName("Tag der Arbeit", (f"{year}-05-01" for year in self.full_range))

    def test_ascension_day(self):
        name = "Auffahrt"
        self.assertHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertHolidayName(name, self.full_range)

    def test_whit_sunday(self):
        name = "Pfingstsonntag"
        self.assertHolidayName(
            name,
            "2020-05-31",
            "2021-05-23",
            "2022-06-05",
            "2023-05-28",
            "2024-05-19",
            "2025-06-08",
        )
        self.assertHolidayName(name, self.full_range)

    def test_whit_monday(self):
        name = "Pfingstmontag"
        self.assertHolidayName(
            name,
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_corpus_christi(self):
        name = "Fronleichnam"
        self.assertHolidayName(
            name,
            "2020-06-11",
            "2021-06-03",
            "2022-06-16",
            "2023-06-08",
            "2024-05-30",
            "2025-06-19",
        )
        self.assertHolidayName(name, self.full_range)

    def test_national_day(self):
        self.assertHolidayName("Staatsfeiertag", (f"{year}-08-15" for year in self.full_range))

    def test_nativity_of_mary(self):
        self.assertHolidayName("Mariä Geburt", (f"{year}-09-08" for year in self.full_range))

    def test_all_saints_day(self):
        self.assertHolidayName("Allerheiligen", (f"{year}-11-01" for year in self.full_range))

    def test_immaculate_conception(self):
        self.assertHolidayName("Mariä Empfängnis", (f"{year}-12-08" for year in self.full_range))

    def test_christmas_eve(self):
        name = "Heiligabend"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(name, (f"{year}-12-24" for year in self.full_range))

    def test_christmas_day(self):
        self.assertHolidayName("Weihnachten", (f"{year}-12-25" for year in self.full_range))

    def test_st_stephens_day(self):
        self.assertHolidayName("Stephanstag", (f"{year}-12-26" for year in self.full_range))

    def test_new_years_eve(self):
        name = "Silvester"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(name, (f"{year}-12-31" for year in self.full_range))

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "Neujahr"),
            ("2022-01-06", "Heilige Drei Könige"),
            ("2022-02-02", "Mariä Lichtmess"),
            ("2022-03-19", "Josefstag"),
            ("2022-04-17", "Ostersonntag"),
            ("2022-04-18", "Ostermontag"),
            ("2022-05-01", "Tag der Arbeit"),
            ("2022-05-26", "Auffahrt"),
            ("2022-06-05", "Pfingstsonntag"),
            ("2022-06-06", "Pfingstmontag"),
            ("2022-06-16", "Fronleichnam"),
            ("2022-08-15", "Staatsfeiertag"),
            ("2022-09-08", "Mariä Geburt"),
            ("2022-11-01", "Allerheiligen"),
            ("2022-12-08", "Mariä Empfängnis"),
            ("2022-12-25", "Weihnachten"),
            ("2022-12-26", "Stephanstag"),
        )

    def test_2022_bank(self):
        self.assertBankHolidaysInYear(
            2022,
            ("2022-01-02", "Berchtoldstag"),
            ("2022-03-01", "Fasnachtsdienstag"),
            ("2022-04-15", "Karfreitag"),
            ("2022-12-24", "Heiligabend"),
            ("2022-12-31", "Silvester"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Neujahr"),
            ("2022-01-02", "Berchtoldstag"),
            ("2022-01-06", "Heilige Drei Könige"),
            ("2022-02-02", "Mariä Lichtmess"),
            ("2022-03-01", "Fasnachtsdienstag"),
            ("2022-03-19", "Josefstag"),
            ("2022-04-15", "Karfreitag"),
            ("2022-04-17", "Ostersonntag"),
            ("2022-04-18", "Ostermontag"),
            ("2022-05-01", "Tag der Arbeit"),
            ("2022-05-26", "Auffahrt"),
            ("2022-06-05", "Pfingstsonntag"),
            ("2022-06-06", "Pfingstmontag"),
            ("2022-06-16", "Fronleichnam"),
            ("2022-08-15", "Staatsfeiertag"),
            ("2022-09-08", "Mariä Geburt"),
            ("2022-11-01", "Allerheiligen"),
            ("2022-12-08", "Mariä Empfängnis"),
            ("2022-12-24", "Heiligabend"),
            ("2022-12-25", "Weihnachten"),
            ("2022-12-26", "Stephanstag"),
            ("2022-12-31", "Silvester"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-02", "Saint Berchtold's Day"),
            ("2022-01-06", "Epiphany"),
            ("2022-02-02", "Candlemas"),
            ("2022-03-01", "Shrove Tuesday"),
            ("2022-03-19", "Saint Joseph's Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-26", "Ascension Day"),
            ("2022-06-05", "Whit Sunday"),
            ("2022-06-06", "Whit Monday"),
            ("2022-06-16", "Corpus Christi"),
            ("2022-08-15", "National Day"),
            ("2022-09-08", "Nativity of Mary"),
            ("2022-11-01", "All Saints' Day"),
            ("2022-12-08", "Immaculate Conception"),
            ("2022-12-24", "Christmas Eve"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Saint Stephen's Day"),
            ("2022-12-31", "New Year's Eve"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-01-02", "День Святого Бертольда"),
            ("2022-01-06", "Богоявлення"),
            ("2022-02-02", "Стрітення"),
            ("2022-03-01", "Масний вівторок"),
            ("2022-03-19", "День Святого Йосипа"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-17", "Великдень"),
            ("2022-04-18", "Великодній понеділок"),
            ("2022-05-01", "День праці"),
            ("2022-05-26", "Вознесіння Господнє"),
            ("2022-06-05", "Трійця"),
            ("2022-06-06", "День Святого Духа"),
            ("2022-06-16", "Свято Тіла і Крові Христових"),
            ("2022-08-15", "Національне свято"),
            ("2022-09-08", "Різдво Пресвятої Богородиці"),
            ("2022-11-01", "День усіх святих"),
            ("2022-12-08", "Непорочне зачаття Діви Марії"),
            ("2022-12-24", "Святий вечір"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "День Святого Стефана"),
            ("2022-12-31", "Переддень Нового року"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import APR
from holidays.groups import InternationalHolidays, IslamicHolidays
from holidays.holiday_base import HolidayBase


class Tajikistan(HolidayBase, InternationalHolidays, IslamicHolidays):
    """Tajikistan holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Tajikistan>
        * [Law #3 from Nov 3, 1995](https://web.archive.org/web/20250823162418/http://www.portali-huquqi.tj/publicadliya/view_qonunhoview.php?showdetail=&asosi_id=155&language=tj)
        * [Law #753 from Aug 2, 2011](https://web.archive.org/web/20250823162543/http://www.portali-huquqi.tj/publicadliya/view_qonunhoview.php?showdetail=&asosi_id=12997&language=tj)
        * [Independence Day](https://web.archive.org/web/20250324203223/https://tnu.tj/index.php/en/9-september-independence-day-of-the-republic-of-tajikistan/)
    """

    country = "TJ"
    default_language = "tg"
    # %s (estimated).
    estimated_label = tr("%s (таҳминан)")
    start_year = 1992
    supported_languages = ("en_US", "ru", "tg")

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=TajikistanIslamicHolidays, show_estimated=islamic_show_estimated
        )
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Соли Нав"))

        # Mother's Day.
        self._add_holiday_mar_8(tr("Рӯзи Модар"))

        # International Nowruz Day.
        name = tr("Иди байналмилалии Наврӯз")
        self._add_holiday_mar_21(name)
        self._add_holiday_mar_22(name)

        # Established by Law #13 from May 3, 2002.
        if self._year >= 2003:
            self._add_holiday_mar_23(name)

        # Established by Law #146 from Dec 28, 2005.
        if self._year >= 2006:
            self._add_holiday_mar_24(name)

        # Abolished by Law #1390 from Feb 24, 2017.
        if self._year <= 2016:
            # International Workers' Solidarity Day.
            self._add_labor_day(tr("Рӯзи байналхалқии якдилии меҳнаткашон"))

        self._add_world_war_two_victory_day(
            # Victory Day.
            tr("Рӯзи Ғалаба дар Ҷанги Бузурги Ватанӣ"),
            is_western=False,
        )

        # Established by Law #628 from May 22, 1998.
        if self._year >= 1998:
            # Day of National Unity.
            self._add_holiday_jun_27(tr("Рӯзи Ваҳдати миллӣ"))

        # Independence Day.
        self._add_holiday_sep_9(tr("Рӯзи Истиқлолияти давлатии Ҷумҳурии Тоҷикистон"))

        if self._year >= 1994:
            # Constitution Day.
            self._add_holiday_nov_6(tr("Рӯзи Конститутсияи Ҷумҳурии Тоҷикистон"))

        # Eid al-Fitr.
        self._add_eid_al_fitr_day(tr("Рӯзи иди Рамазон"))

        # Eid al-Adha.
        self._add_eid_al_adha_day(tr("Рӯзи иди Қурбон"))


class TJ(Tajikistan):
    pass


class TJK(Tajikistan):
    pass


class TajikistanIslamicHolidays(_CustomIslamicHolidays):
    # https://web.archive.org/web/20240911001624/https://www.timeanddate.com/holidays/tajikistan/eid-al-fitr
    EID_AL_FITR_DATES_CONFIRMED_YEARS = (1992, 2025)
    EID_AL_FITR_DATES = {
        2023: (APR, 22),
    }

    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (1992, 2025)



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.mauritius import Mauritius
from tests.common import CommonCountryTests


class TestMauritius(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(1988, 2050)
        super().setUpClass(Mauritius, years=years)
        cls.no_estimated_holidays = Mauritius(years=years, islamic_show_estimated=False)

    def test_special_holidays(self):
        self.assertHolidayName("Public Holiday", "2019-07-29", "2019-09-09")

    def test_new_years_day(self):
        self.assertHolidayName("New Year's Day", (f"{year}-01-01" for year in range(1988, 2050)))

    def test_day_after_new_years_day(self):
        self.assertHolidayName(
            "Day after New Year's Day", (f"{year}-01-02" for year in range(1988, 2050))
        )

    def test_abolition_of_slavery_day(self):
        self.assertHolidayName(
            "Abolition of Slavery", (f"{year}-02-01" for year in range(1988, 2050))
        )

    def test_independence_and_republic_day(self):
        self.assertHolidayName(
            "Independence and Republic Day", (f"{year}-03-12" for year in range(1988, 2050))
        )

    def test_labour_day(self):
        self.assertHolidayName("Labour Day", (f"{year}-05-01" for year in range(1988, 2050)))

    def test_assumption_day(self):
        name = "Assumption of the Blessed Virgin Mary"
        self.assertHolidayName(name, (f"{year}-08-15" for year in range(2016, 2050, 2)))
        self.assertNoHolidayName(name, range(1988, 2016), range(2017, 2050, 2))

    def test_all_saints_day(self):
        name = "All Saints' Day"
        self.assertHolidayName(
            name, (f"{year}-11-01" for year in (*range(1988, 2016), *range(2017, 2050, 2)))
        )
        self.assertNoHolidayName(name, range(2016, 2050, 2))

    def test_arrival_of_indentured_labourers_day(self):
        self.assertHolidayName(
            "Arrival of Indentured Labourers", (f"{year}-11-02" for year in range(1988, 2050))
        )

    def test_christmas_day(self):
        self.assertHolidayName("Christmas Day", (f"{year}-12-25" for year in range(1988, 2050)))

    def test_chinese_spring_festival(self):
        name = "Chinese Spring Festival"
        self.assertHolidayName(
            name,
            "2021-02-12",
            "2022-02-01",
            "2023-01-22",
            "2024-02-10",
            "2025-01-29",
        )
        self.assertHolidayName(name, range(1988, 2050))

    def test_eid_al_fitr(self):
        name = "Eid-ul-Fitr"
        self.assertHolidayName(
            name,
            "2021-05-14",
            "2022-05-03",
            "2023-04-22",
            "2024-04-11",
            "2025-04-01",
        )
        self.assertHolidayName(name, self.no_estimated_holidays, range(1988, 2050))

    def test_thaipusam(self):
        name = "Thaipoosam Cavadee"
        self.assertHolidayName(
            name,
            "2021-01-28",
            "2022-01-18",
            "2023-02-04",
            "2024-01-25",
            "2025-02-11",
        )
        self.assertHolidayName(name, range(1988, 2050))

    def test_maha_shivaratri(self):
        name = "Maha Shivaratree"
        self.assertHolidayName(
            name,
            "2021-03-11",
            "2022-03-01",
            "2023-02-18",
            "2024-03-08",
            "2025-02-26",
        )
        self.assertHolidayName(name, range(2001, 2036))

    def test_ougadi(self):
        name = "Ougadi"
        self.assertHolidayName(
            name,
            "2021-04-13",
            "2022-04-02",
            "2023-03-22",
            "2024-04-09",
            "2025-03-30",
        )
        self.assertHolidayName(name, range(2001, 2036))

    def test_ganesh_chaturthi(self):
        name = "Ganesh Chaturthi"
        self.assertHolidayName(
            name,
            "2021-09-11",
            "2022-09-01",
            "2023-09-20",
            "2024-09-08",
            "2025-08-28",
        )
        self.assertHolidayName(name, range(2001, 2036))

    def test_diwali(self):
        name = "Divali"
        self.assertHolidayName(
            name,
            "2021-11-04",
            "2022-10-24",
            "2023-11-12",
            "2024-10-31",
            "2025-10-20",
        )
        self.assertHolidayName(name, range(2001, 2036))

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "New Year's Day"),
            ("2024-01-02", "Day after New Year's Day"),
            ("2024-01-25", "Thaipoosam Cavadee"),
            ("2024-02-01", "Abolition of Slavery"),
            ("2024-02-10", "Chinese Spring Festival"),
            ("2024-03-08", "Maha Shivaratree"),
            ("2024-03-12", "Independence and Republic Day"),
            ("2024-04-09", "Ougadi"),
            ("2024-04-11", "Eid-ul-Fitr"),
            ("2024-05-01", "Labour Day"),
            ("2024-08-15", "Assumption of the Blessed Virgin Mary"),
            ("2024-09-08", "Ganesh Chaturthi"),
            ("2024-10-31", "Divali"),
            ("2024-11-02", "Arrival of Indentured Labourers"),
            ("2024-12-25", "Christmas Day"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "New Year's Day"),
            ("2025-01-02", "Day after New Year's Day"),
            ("2025-01-29", "Chinese Spring Festival"),
            ("2025-02-01", "Abolition of Slavery"),
            ("2025-02-11", "Thaipoosam Cavadee"),
            ("2025-02-26", "Maha Shivaratree"),
            ("2025-03-12", "Independence and Republic Day"),
            ("2025-03-30", "Ougadi"),
            ("2025-04-01", "Eid-ul-Fitr"),
            ("2025-05-01", "Labour Day"),
            ("2025-08-28", "Ganesh Chaturthi"),
            ("2025-10-20", "Divali"),
            ("2025-11-01", "All Saints' Day"),
            ("2025-11-02", "Arrival of Indentured Labourers"),
            ("2025-12-25", "Christmas Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-01-02", "Day after New Year's Day"),
            ("2025-01-29", "Chinese Spring Festival"),
            ("2025-02-01", "Abolition of Slavery"),
            ("2025-02-11", "Thaipusam"),
            ("2025-02-26", "Maha Shivaratri"),
            ("2025-03-12", "Independence and Republic Day"),
            ("2025-03-30", "Ugadi"),
            ("2025-04-01", "Eid al-Fitr"),
            ("2025-05-01", "Labor Day"),
            ("2025-08-28", "Ganesh Chaturthi"),
            ("2025-10-20", "Diwali"),
            ("2025-11-01", "All Saints' Day"),
            ("2025-11-02", "Arrival of Indentured Laborers"),
            ("2025-12-25", "Christmas Day"),
        )
